function [ y0, dydx, U_dydx ] = LineFitFunc( x, y, U_y )
% LineFitFunc This function will take the vectors x and y, perform Linear Regression, and output the y-value
%   at the wall y0, the gradient dydx, and the uncertainty in the gradient U_dydx. The fit is weighted to
%   values that have lower uncertainty. The method is from Bevington, P. R. and Robinson, D. K., "Data
%   Reduction and Error Analysis for the Physical Sciences," McGraw Hill, New York, 2003, 3rd Ed., pg 109.

% Check if uncertainty is 0 and replace with small number
% U_y(find(~U_y)) = 1E-4;
U_y(~U_y) = 1E-3;

% Perform Linear Regression for Gradient
Delta = sum(1./U_y.^2)*sum(x.^2./U_y.^2) - sum(x./U_y.^2)^2;
% y-intercept
y0 = 1/Delta*( sum(x.^2./U_y.^2)*sum(y./U_y.^2) - sum(x./U_y.^2)*sum(x.*y./U_y.^2) );
% Gradient
dydx = 1/Delta*( sum(1./U_y.^2)*sum(x.*y./U_y.^2) - sum(x./U_y.^2)*sum(y./U_y.^2) );
% Uncertainty of gradient
U_dydx = sqrt(1/Delta*sum(1./U_y.^2));

end