clear all; close all; clc; dbstop if error;

% This driver code can be used instead of PIVuncertaintyGUI.m to call the PIVuncertaintyCode.m function. This
% was done for the Transient data since the number of data sets is so large. Another reason for this driver is
% that the dt of the particle images may be different within a data set.  This code will run for all phases of
% one case.  It requires the uigetfile_n_dir.m function to select multiple folders.  It can be used for data
% sets within the same folder with any names, but requires the paths under the names to be equivalent.  It
% will create folders for the output files next to the *.vc7 files.
%
% Blake Lance
% 11 December 2014

%% Inputs

JobName = 'S8_2';
Parallel = '1'; % Use parallel=1, not parallel=0
ParProcs = '5'; % The number of parallel jobs

CaseDir = uigetfile_n_dir('H:\','Select all the data set folders to investigate');
[ImgFile,ImgDir] = uigetfile('*.im7','Select the first *.im7',CaseDir{1});
[VecFile,VecDir] = uigetfile('*.vc7','Select the first *.vc7',ImgDir);
[SurfFile,SurfDir] = uigetfile('*.mat','Select the uncertainty surface to use');

tic;

%% Process through Phases in a Loop

Nphases = length(CaseDir);
for i=1:Nphases  % Loop through the phase folders
    SlashLoc = strfind(CaseDir{i},'\'); % Find the locations of '\' in the data sets
    SlashLoc = SlashLoc(end);   % Use the last one
    phase = CaseDir{i};
    phase = num2str(phase(SlashLoc+1:end));   % Get the data set name

    % Count the number of files that are *.im7s
    Nimages = 0; % The number of image pairs
    PhaseDir = dir(CaseDir{i});
    for j=1:length(PhaseDir);
        if length(PhaseDir(j).name) >= 3    % Skip '.' and '..' since they cause problems
            if strcmp(PhaseDir(j).name(end-2:end),'im7')    % The file is an im7
                Nimages = Nimages + 1;
            end
        end
    end
    
    % Replace the case name for the current phase
    ImgDir_new = strrep(ImgDir,CaseDir{1},CaseDir{i});
    VecDir_new = strrep(VecDir,CaseDir{1},CaseDir{i});
    
    % Create Output Directory next to vc7s
    mkdir([VecDir_new,'PIV Uncertainty']);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Assemble the Data struct to pass into PIVuncertainty
    
    % Reading information
    Data.imdirec = ImgDir_new(1:end-1); % Take off the '\'
    Data.imbase = 'B';
    Data.imzeros = '5';
    Data.imext = 'im7';
    Data.imfstep = '1';
    Data.imfstart = '1';
    Data.imfend = num2str(Nimages);

    Data.datadirec = VecDir_new(1:end-1); % Take off the '\'
    Data.database = 'B';
    Data.datazeros = '5';
    Data.dataext = 'vc7';
    Data.datacstep = '0';
    Data.datafstep = '1';
    Data.datafstart = '1';
    Data.datafend = num2str(Nimages);
    
    Data.surfdirec = SurfDir(1:end-1); % Take off the '\'
    Data.surfname = SurfFile;
    Data.batchname = [JobName,'_',phase];   % Have the code find the phase
    Data.bkgd = '1';
    
    Data.Cor = 'radiobutton2';
    Data.cpass = '1';
    Data.DiaGS = '2';
    Data.DenGS = '2';
    Data.denlimit = '0';
    Data.dens = '';
    Data.diam = '';
    Data.CalcDia = '1';
    Data.CalcDen = '1';
    Data.hardDens = '0';
    Data.hardDia = '0';
    Data.imNum = '1';
    Data.multifolders = '0';
    Data.matout = '1';
    Data.minDen = '';
    Data.outbase = 'B';
    Data.outdirec = [VecDir_new,'PIV Uncertainty'];
    Data.par = Parallel;
    Data.parprocessors = ParProcs;
    Data.passes = '1';
    Data.pltout = '0';
    Data.stats = '1';
    Data.statsOnly = '0';
    Data.vcNum = '1';
    Data.wrmag = '1';
    Data.wrsep = '1';

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Save the job file
    save([Data.outdirec '\' Data.batchname '_Job.mat'],'Data');
    
    % Call PIVuncertainty
    PIVuncertaintyCode(Data);

    % Calculate stats
    PIV_Stats(Data);
end

toc;