clear all; close all; clc;

% This program will read in the spatially-averaged Vy across the different runs within a Transient data set
% from DaVis in TecPlot format.  It checks for outliers by Chauvenet's criterion and returns which run(s), if
% any, should be removed from the ensemble average.
%
% Blake Lance
% 11 November 2014

%% Inputs

% Have the user select the TecPlot file for the case in question
[VyFile,p] = uigetfile('*.dat','Select the Vy TecPlot file for this case');
data = importdata([p,VyFile]);

% Some Stats
Vy = data.data(:,2);
Vy_ave = mean(Vy);
Vy_SD = std(Vy);

% Chauvenet's Criterion (interpolates for 50-300 samples)
tau = abs( ( Vy - Vy_ave )/Vy_SD );
C_tau_array = [50 2.57; 100 2.81; 300 3.14];
C_tau = interp1(C_tau_array(:,1),C_tau_array(:,2),length(Vy));

% Find the indeces where the data are outliers
indeces = find( tau > C_tau );

% Reduce the data and compare the mean and standard deviation
Vy(indeces) = NaN;
Vy_new = Vy(isfinite(Vy));
Vy_new_ave = mean(Vy_new);
Vy_new_SD = std(Vy_new);

% Produce Outputs
fprintf('The Transient runs with outliers are %i \n',indeces);
fprintf('The new mean is %5.2f%% different from the old \n',( Vy_new_ave - Vy_ave )/Vy_ave*100);
fprintf('The new standard deviation is %7.2f%% different from the old \n',( Vy_new_SD - Vy_SD )/Vy_SD*100);
