function surfaceViewer
% Gui to Visualize Uncertainty Surface
% Created: 18 Dec 2014 - Rick Cressall
% Changelog:
% 22 Dec 2014 - RMS of Bias and Random now correct
% 29 Dec 2014 - Added support for 4D surfaces

    close all;
    warning off;
    pushButtonWidth = .07;
    pushButtonHeight = .06;
    figX = 100;
    figY = 90;
    figDelX = 1200;
    figDelY = 800;    
    
    h.f = figure('Position',[figX figY figDelX figDelY]);    
    set(h.f,'Toolbar','figure');
    h.paramArray = [0 0 1 1 1];
    h.paramNames = {'density','diameter','shear','displacement','Fo'};
    h.paramLabel = {'sDensity','sDiameter','sShear','sDisplacement','sOOP'};
    h.dispLabels = {'Density','Diameter','Shear','Displacement','Out of Plane'};

    h.axes = axes(  'Units','norm',...
                    'Position',[.07 .20 .6 .75]);
    
    pbLoadSurface = uicontrol('Style','pushbutton',...
                                'units','norm',...
                                'String','Load Surface',...
                                'Callback',{@pbLoadSurface_cb},...
                                'Position',[.92 .05 .07 .05]);  

    % Uncertainty Type
        h.uncertParam = uipanel('Position',[.69 .20 .3 .15]);
        h.cbBias = uicontrol('parent',h.uncertParam,...
                             'Style','checkbox',...
                             'units','norm',...
                             'String','Bias Uncertainty',...
                             'Value',1,...
                             'Callback',{@uncertParam_cb},...
                             'Position',[.1 .8 .4 .15]);
        h.randomPanel = uibuttongroup('parent',h.uncertParam,...
                                      'units','norm',...
                                      'Title','Random Uncertainty',...                                  
                                      'Position',[.5 .1 .4 .85]);                              
            h.randomPanelTitle = get(h.randomPanel,'TitleHandle');
            newPos = get(h.randomPanelTitle,'position') + [0,0,20,0];
            set(h.randomPanelTitle, 'style','checkbox','pos',newPos, ...
                    'Callback',{@uncertParam_cb},'Value',1);        
        h.cbRhigh = uicontrol('parent',h.randomPanel,...
                              'Style','radio',...
                              'units','norm',...
                              'String','R_High',...
                              'Value',1,...
                              'Position',[.1 .7 .8 .3]);
        h.cbRlow = uicontrol('parent',h.randomPanel,...
                             'Style','radio',...
                             'units','norm',...
                             'String','R_Low',...
                             'Position',[.1 .4 .8 .3]);
                                    
    % Parameter Panel
        panelParam = uipanel('Position',[.69 .40 .3 .55],'units','norm');
    
    % Check Boxes
        h.cbDensity = uicontrol(  'parent',panelParam,...
                                'Style','checkbox',...
                                'units','norm',...
                                'String','Density',...
                                'Callback',{@cbParam_cb},...
                                'Position',[.1 .9 .4 .1]);    
        h.cbDiameter = uicontrol(  'parent',panelParam,...
                                'Style','checkbox',...
                                'units','norm',...
                                'String','Diameter',...
                                'Callback',{@cbParam_cb},...
                                'Position',[.1 .7 .4 .1]);                            
        h.cbDisplacement = uicontrol(  'parent',panelParam,...
                                'Style','checkbox',...
                                'units','norm',...
                                'String','Displacement',...
                                'Callback',{@cbParam_cb},...
                                'Value',1,...
                                'Position',[.1 .5 .4 .1]);                            
        h.cbOOP = uicontrol(  'parent',panelParam,...
                                'Style','checkbox',...
                                'units','norm',...
                                'String','Out of Plane Motion',...
                                'Callback',{@cbParam_cb},...
                                'Value',1,...
                                'Position',[.1 .3 .4 .1]);                            
        h.cbShear = uicontrol(  'parent',panelParam,...
                                'Style','checkbox',...
                                'units','norm',...
                                'String','Shear',...
                                'Value',1,...
                                'Callback',{@cbParam_cb},...
                                'Position',[.1 .1 .4 .1]);                            
    
    % Slider Bars
        h.sDensity = uicontrol('parent',panelParam,...
                            'Style','slider',...
                            'units','norm',...
                            'Callback',{@slider_cb},...
                            'String','density',...
                            'Position',[.1 .87 .8 .05]);                        
        h.sDiameter = uicontrol('parent',panelParam,...
                            'Style','slider',...
                            'units','norm',...
                            'Callback',{@slider_cb},...
                            'String','diameter',...
                            'Position',[.1 .67 .8 .05]);                        
        h.sDisplacement = uicontrol('parent',panelParam,...
                            'Style','slider',...
                            'units','norm',...
                            'Callback',{@slider_cb},...
                            'String','displacement',...
                            'Position',[.1 .47 .8 .05]);                        
        h.sOOP = uicontrol('parent',panelParam,...
                            'Style','slider',...
                            'units','norm',...
                            'Callback',{@slider_cb},...
                            'String','Fo',...
                            'Position',[.1 .27 .8 .05]);                        
        h.sShear = uicontrol('parent',panelParam,...
                            'Style','slider',...
                            'units','norm',...
                            'Callback',{@slider_cb},...
                            'String','shear',...
                            'Position',[.1 .07 .8 .05]);
                        
   % Min
       h.minDensity = uicontrol('parent',panelParam,...
                                'Style','text',...
                                'String','0',...
                                'units','norm',...
                                'Position',[.1 .83 .1 .03]);
       h.minDiameter = uicontrol('parent',panelParam,...
                                'Style','text',...
                                'String','0',...
                                'units','norm',...
                                'Position',[.1 .63 .1 .03]);
       h.minDisplacement = uicontrol('parent',panelParam,...
                                'Style','text',...
                                'String','0',...
                                'units','norm',...
                                'Position',[.1 .43 .1 .03]);
       h.minOOP = uicontrol('parent',panelParam,...
                                'Style','text',...
                                'String','0',...
                                'units','norm',...
                                'Position',[.1 .23 .1 .03]);
       h.minShear = uicontrol('parent',panelParam,...
                                'Style','text',...
                                'String','0',...
                                'units','norm',...
                                'Position',[.1 .03 .1 .03]);
                            
    % Max
        h.maxDensity = uicontrol('parent',panelParam,...
                                'Style','text',...
                                'String','1',...
                                'units','norm',...
                                'Position',[.8 .83 .1 .03]);
        h.maxDiameter = uicontrol('parent',panelParam,...
                                'Style','text',...
                                'String','1',...
                                'units','norm',...
                                'Position',[.8 .63 .1 .03]);
        h.maxDisplacement = uicontrol('parent',panelParam,...
                                'Style','text',...
                                'String','1',...
                                'units','norm',...
                                'Position',[.8 .43 .1 .03]);
        h.maxOOP = uicontrol('parent',panelParam,...
                                'Style','text',...
                                'String','1',...
                                'units','norm',...
                                'Position',[.8 .23 .1 .03]);
        h.maxShear = uicontrol('parent',panelParam,...
                                'Style','text',...
                                'String','1',...
                                'units','norm',...
                                'Position',[.8 .03 .1 .03]);                            

    % Control z axis
        h.cbControlZ = uicontrol('Style','Checkbox',...
                                 'String','Manual Z Axis',...
                                 'units','norm',...
                                 'Callback',{@cbControlZ_cb},...
                                 'Position',[.01 .10 .1 .03]);
        set(h.cbControlZ,'backgroundcolor',get(h.f,'color'));
        h.sControlZ = uicontrol('Style','slider',...
                                'units','norm',...
                                'Callback',{@sControlZ_cb},...
                                'Create',{@sControlZ_cr},...
                                'enable','off',...
                                'Position',[.015 .20 .02 .75]);
        h.textZmax = uicontrol('Style','edit',...
                               'units','norm',...
                               'enable','on',...
                               'String','1',...
                               'Callback',{@textZlim_cb},...
                               'Position',[.01 .96 .03 .02]);
        h.textZmin = uicontrol('Style','edit',...
                               'units','norm',...
                               'enable','on',...
                               'String','0',...
                               'Callback',{@textZlim_cb},...
                               'Position',[.01 .17 .03 .02]);
        addlistener(h.sControlZ,'Value','PreSet',@sControlZ_cb);
                            
        disableBoxes();
%% Create Functions
    function sControlZ_cr(hObject,eventdata)
        h.ControlZ = 0; % Not manually controlling Z axis
    end

%% Callback Functions

    function pbLoadSurface_cb(hObject, eventdata)
        [fileName,filePath] = uigetfile('*.mat','Choose Uncertainty Surface .mat File');        
        h.data = load([filePath fileName]);
        
        % Determine if surface is 4D or 5D (Out of plane Motion)
        h.dim = length(size(h.data.b_k));
        if ( h.dim ~= 5 && h.dim~= 4 )
            err = MException('SurfaceViewer:IncorrectSurfaceDimensions', ...
                            'Surface Must be 4 or 5 Dimensions');
            throw(err);
        end
        
        % Only load Out of plane motion (variable Fo) if 5D
        if( h.dim == 5 )
            set(h.minOOP,'String',num2str(min(unique(h.data.Fo))));
            set(h.maxOOP,'String',num2str(max(unique(h.data.Fo))));
            N = length(unique(h.data.Fo))-1;
            set(h.sOOP,'SliderStep',[1/N, 1/N]);
            h.realVals.Fo = h.data.Fo(1);
            h.realIdx.Fo = 1;
            set(h.sOOP,'Enable','on');
            set(h.cbOOP,'Enable','on');     
            h.paramArray(5) = 1;
            h.updatePlotParam = @updatePlotParam5D;
        else
            set(h.cbOOP,'Value',0);
            h.paramArray(5) = 0;
            h.updatePlotParam = @updatePlotParam4D;
        end
        
        
        % Update Min and Max Values
        set(h.minDensity,       'String',num2str(min(unique(h.data.density))));
        set(h.minDiameter,      'String',num2str(min(unique(h.data.diameter))));
        set(h.minShear,         'String',num2str(min(unique(h.data.shear))));
        set(h.minDisplacement,  'String',num2str(min(unique(h.data.displacement))));        
        
        set(h.maxDensity,       'String',num2str(max(unique(h.data.density))));
        set(h.maxDiameter,      'String',num2str(max(unique(h.data.diameter))));
        set(h.maxDisplacement,  'String',num2str(max(unique(h.data.displacement))));
        set(h.maxShear,         'String',num2str(max(unique(h.data.shear))));
        
        
        % Update Sliders
        N = length(unique(h.data.density))-1;
        set(h.sDensity,'SliderStep',[1/N, 1/N]);
        h.realVals.density = h.data.density(1);
        h.realIdx.density = 1;

        N = length(unique(h.data.diameter))-1;
        set(h.sDiameter,'SliderStep',[1/N, 1/N]);
        h.realVals.diameter = h.data.diameter(1);
        h.realIdx.diameter = 1;

        N = length(unique(h.data.shear))-1;
        set(h.sShear,'SliderStep',[1/N, 1/N]);
        h.realVals.shear = h.data.shear(1);
        h.realIdx.shear = 1;

        N = length(unique(h.data.displacement))-1;
        set(h.sDisplacement,'SliderStep',[1/N, 1/N]);
        h.realVals.displacement = h.data.displacement(1);
        h.realIdx.displacement = 1;
       
        
        set(h.cbBias,'Enable','on');
        set(h.randomPanelTitle,'Enable','on');
        set(h.cbRhigh,'Enable','on');
        set(h.cbRlow,'Enable','on');
        set(h.cbDensity,'Enable','on');
        set(h.cbDiameter,'Enable','on');
        set(h.cbDisplacement,'Enable','on');        
        set(h.cbShear,'Enable','on');
        set(h.sDensity,'Enable','on');
        set(h.sDiameter,'Enable','on');
        set(h.sDisplacement,'Enable','on');        
        set(h.sShear,'Enable','on');
        set(h.cbControlZ,'Enable','on');
     
        h.updatePlotParam();
    end

    function cbParam_cb(hObject, eventdata)
        switch(get(hObject,'String'))
            case 'Density'
                h.paramArray(1) = ~h.paramArray(1);
            case 'Diameter'
                h.paramArray(2) = ~h.paramArray(2);
            case 'Shear'
                h.paramArray(3) = ~h.paramArray(3);
            case 'Displacement'
                h.paramArray(4) = ~h.paramArray(4);
            case 'Out of Plane Motion'
                h.paramArray(5) = ~h.paramArray(5);
        end
%         display(get(hObject,'String'));
%         display(h.paramArray); 
        h.updatePlotParam();
    end

    function slider_cb(hObject, eventdata)
        possibleVals = eval(['unique(h.data.' get(hObject,'String') ');']);
        N = length(possibleVals)-1;
        curVal = get(hObject,'Value');
        eval(['h.realVals.' get(hObject,'String') ' = possibleVals(round(curVal*N)+1);']);
        eval(['h.realIdx.' get(hObject,'String') ' = round(curVal*N)+1;']);        
        h.updatePlotParam();
    end

    function uncertParam_cb(hObject, eventdata)
        h.updatePlotParam();        
    end

    function cbControlZ_cb(hObject,eventdata)
        if( strcmp(get(h.sControlZ,'Enable'),'off') )
            set(h.sControlZ,'Enable','on');
            h.ControlZ = 1;
            h.Zaxis = zlim;         % Grab current z limits
            h.Zmax = 5*h.Zaxis(2);  % Set max to 5x the current max
            h.Zmin = h.Zaxis(1);    % Set min to current min            
            set(h.sControlZ,'max',h.Zmax);
            set(h.sControlZ,'min',h.Zmin);
            set(h.sControlZ,'value',h.Zaxis(2));
            set(h.textZmax,'String',h.Zmax);
            set(h.textZmin,'String',h.Zmin);
        else
            set(h.sControlZ,'Enable','off');
            set(h.textZmax,'String',h.Zmax);
            set(h.textZmin,'String',h.Zmin);
            h.ControlZ = 0;
        end
    end

    function sControlZ_cb(hObject,eventdata)
%         zlim([h.Zmin get(hObject,'Value')]);
%         caxis([h.Zmin get(hObject,'Value')]);
        updatePlot();
    end

    function textZlim_cb(hObject,eventdata)
        temp = str2num(get(h.textZmax,'String'));
        if ( isnumeric(temp) && temp > h.Zmin )
            h.Zmax = temp;
            set(h.textZmax,'String',temp);
            set(h.sControlZ,'max',temp);
            display('Setting Z Max');
        end        
        temp = get(h.textZmin,'String');
        if ( isnumeric(temp) && temp < h.Zmax )
            h.Zmin = temp;
            set(h.textZmin,'String',temp);
            set(h.sControlZ,'min',temp);
        end            
    end


%% Draw Functions
    function updatePlotParam5D()
        if (sum(h.paramArray) ~= 3)
            display('Need to select 3 dimensions to hold constant.');
            return; % Don't do anything, need 3 marks
        else
            % Find the 3 chosen variables
            surfIndex = find(h.paramArray);
            axisIndex = find(~h.paramArray);
            
            % Build String for surf plot
                if( get(h.cbDensity,'Value') )                    
                    s1 = num2str(h.realIdx.density);
                else
                    s1 = ':';
                end
                if( get(h.cbDiameter,'Value') )
                    s2 = num2str(h.realIdx.density);
                else
                    s2 = ':';
                end
                if( get(h.cbShear,'Value') )
                    s3 = num2str(h.realIdx.shear);
                else
                    s3 = ':';
                end
                if( get(h.cbDisplacement,'Value') )
                    s4 = num2str(find(unique(h.data.displacement)==get(h.sDisplacement,'Value')));
                else
                    s4 = ':';
                end
                if( get(h.cbOOP,'Value') )
                    s5 = num2str(h.realIdx.Fo);
                else
                    s5 = ':';
                end         
                h.dimString = ['(' s1 ',' s2 ',' s3 ',' s4 ',' s5 ')'];
   
                title1 = [h.dispLabels{surfIndex(1)} ': ' ...
                    num2str(eval(['h.realVals.' h.paramNames{surfIndex(1)} ';']))];
                title2 = [h.dispLabels{surfIndex(2)} ': ' ...
                    num2str(eval(['h.realVals.' h.paramNames{surfIndex(2)} ';']))];
                title3 = [h.dispLabels{surfIndex(3)} ': ' ...
                    num2str(eval(['h.realVals.' h.paramNames{surfIndex(3)} ';']))];
              

            % Get labels for the two axis
            h.axis1 = [h.paramNames{axisIndex(1)}];
            h.axis2 = [h.paramNames{axisIndex(2)}];
            
            % Get Which Uncertainty to Plot            
            if (get(h.cbBias,'Value') && get(h.randomPanelTitle,'Value'))
            % RMS Error
                if( get(h.cbRhigh,'Value') )
                    h.uncert = sqrt((h.data.b_k).^2+(h.data.r_high).^2);
                else
                    h.uncert = sqrt((h.data.b_k).^2+(h.data.r_high).^2);
                end
                title4 = 'Uncertainty: RMS of Bias and Random';
            elseif( get(h.cbBias,'Value') )
                h.uncert = h.data.b_k;
                title4 = 'Uncertainty: Bias';
            elseif( get(h.randomPanelTitle,'Value') )
                if( get(h.cbRhigh,'Value') )
                    h.uncert = h.data.r_high;
                else
                    h.uncert = h.data.r_low;
                end
                title4 = 'Uncertainty: Random';
            else
                display('Select Uncertainty to Plot');
                return;
            end
            h.titleString = [title4 '  -  ' title1 '  ' title2 '  ' title3];
        end
        
        updatePlot();
    end

    function updatePlotParam4D()
%         display('function call: updatePlotParam()');
        if (sum(h.paramArray) ~= 2)
            display('Need to select 2 dimensions to hold constant.');
            return; % Don't do anything, need 2 marks
        else
            % Find the 2 chosen variables
            surfIndex = find(h.paramArray);
            axisIndex = find(~h.paramArray);
            
            % Build String for surf plot
                if( get(h.cbDensity,'Value') )                    
                    s1 = num2str(h.realIdx.density);
                else
                    s1 = ':';
                end
                if( get(h.cbDiameter,'Value') )
                    s2 = num2str(h.realIdx.diameter);
                else
                    s2 = ':';
                end
                if( get(h.cbShear,'Value') )
                    s3 = num2str(h.realIdx.shear);
                else
                    s3 = ':';
                end
                if( get(h.cbDisplacement,'Value') )
                    s4 = num2str(h.realIdx.displacement);
                else
                    s4 = ':';
                end
    
                h.dimString = ['(' s1 ',' s2 ',' s3 ',' s4 ')'];

                title1 = [h.dispLabels{surfIndex(1)} ': ' ...
                    num2str(eval(['h.realVals.' h.paramNames{surfIndex(1)} ';']))];
                title2 = [h.dispLabels{surfIndex(2)} ': ' ...
                    num2str(eval(['h.realVals.' h.paramNames{surfIndex(2)} ';']))];
                

            % Get labels for the two axis
            h.axis1 = [h.paramNames{axisIndex(1)}];
            h.axis2 = [h.paramNames{axisIndex(2)}];
            
            % Get Which Uncertainty to Plot            
            if (get(h.cbBias,'Value') && get(h.randomPanelTitle,'Value'))
            % RMS Error
                if( get(h.cbRhigh,'Value') )
                    h.uncert = sqrt((h.data.b_k).^2+(h.data.r_high).^2);
                else
                    h.uncert = sqrt((h.data.b_k).^2+(h.data.r_low).^2);
                end
                title4 = 'Uncertainty: RMS of Bias and Random';
            elseif( get(h.cbBias,'Value') )
                h.uncert = h.data.b_k;
                title4 = 'Uncertainty: Bias';
            elseif( get(h.randomPanelTitle,'Value') )
                if( get(h.cbRhigh,'Value') )
                    h.uncert = h.data.r_high;
                else
                    h.uncert = h.data.r_low;
                end
                title4 = 'Uncertainty: Random';
            else
                display('Select Uncertainty to Plot');
                return;
            end
            h.titleString = [title4 '  -  ' title1 '  ' title2];
        end
        
        updatePlot();
    end

    function updatePlot()
        h.axes = gca;
        [az, el] = view;
            
        if (sum(h.paramArray) == 3 && h.dim == 5)
            clear gca;         
            eval(['surf(squeeze(h.data.' h.axis1 h.dimString ...
                     '),squeeze(h.data.' h.axis2 h.dimString ...
                     '),squeeze(h.uncert' h.dimString '));'])
            title(h.titleString);
            xlabel(h.axis1);
            ylabel(h.axis2);
            if (az == 0 && el == 90)
                view(3);
                else
                view(az,el);
            end

            if ( h.ControlZ )
                max = get(h.sControlZ,'Value');
                if( max > h.Zmin )
                    zlim([h.Zmin max]);
                    caxis([h.Zmin max]);
                end
                else
                temp = zlim;
                set(h.textZmax,'String',temp(2));
                set(h.textZmin,'String',temp(1));
            end
        elseif( sum(h.paramArray) == 2 && h.dim == 4)
            clear gca;         
            eval(['surf(squeeze(h.data.' h.axis1 h.dimString ...
                     '),squeeze(h.data.' h.axis2 h.dimString ...
                     '),squeeze(h.uncert' h.dimString '));'])
            title(h.titleString);
            xlabel(h.axis1);
            ylabel(h.axis2);
            if (az == 0 && el == 90)
                view(3);
                else
                view(az,el);
            end

            if ( h.ControlZ )
                zlim([h.Zmin get(h.sControlZ,'Value')]);
                caxis([h.Zmin get(h.sControlZ,'Value')]);
                else
                temp = zlim;
                set(h.textZmax,'String',temp(2));
                set(h.textZmin,'String',temp(1));
            end
        end
        rotate3d on;
    end
    
    function disableBoxes()
        set(h.cbBias,'Enable','off');
        set(h.randomPanelTitle,'Enable','off');
        set(h.cbRhigh,'Enable','off');
        set(h.cbRlow,'Enable','off');
        set(h.cbDensity,'Enable','off');
        set(h.cbDiameter,'Enable','off');
        set(h.cbDisplacement,'Enable','off');
        set(h.cbOOP,'Enable','off');
        set(h.cbShear,'Enable','off');
        set(h.sDensity,'Enable','off');
        set(h.sDiameter,'Enable','off');
        set(h.sDisplacement,'Enable','off');
        set(h.sOOP,'Enable','off');
        set(h.sShear,'Enable','off');
        set(h.cbControlZ,'Enable','off');
    end

end