clear all; close all; clc;format long;
%% Shaun Harris wrote this code Dec. 19, 2013
% email: shaun.r.harris@gmail.com
% feel free to contact if you have many questions

% Version 2_1, Blake Lance, 3 June 2015
%   Updated headers with units, renamed output files
% Version 2_2, Blake Lance, 3 June 2015
%   Removed output of standard deviation, added bias and total uncertainties at 95% confident level
%       The bias of the temperature measurements is 1degC and of heat flux is 5%
%   Reduced output precision from 7 to 4 digits
%   Changed heat flux output to csv file
%   Changed header of old precision uncertainty P to random uncertainty S

%% User Input
% put a 0 if you have 6 .csv files in one folder for your case or
% put a 1 if you have 5 temperature files in one folder and the heat flux
Filetype=input('Put a 0 if you have all 6 .csv files in one folder for your case \n Put a 1 if you have 5 temperature files in one spot and the heat flux files in another \n'); % change this number based on how your data is saved
%% if 0... input after running code is a 0
% 5 temperature .csv files under one folder and flux .csv files somewhere else
% Selecting files:  This program will ask you to select 2 seperate folders
%
% First folder:  Select the folder that is above a series of folders  that
% contain the 5 temperature .csv files
% Should be named HeatedWallTemp.csv, InletTemp.csv, RightWallTemp.csv,
% LeftWallTemp.csv and TopWallTemp.csv
%
% Second folder:  Select a folder that is above a series of .csv heat flux
% files that you want the stats for.  Make sure no other .csv files are in
% there
%
%% if 1... input after running code is a 1
% 6 .csv files in the same folder
% This code will ask you to select one folder.  Select the folder that is 
% above a series of folders  that contain the 6 .csv files HeatedWallTemp.csv,
% InletTemp.csv, and etc. in each of the folders.  Make sure no other folders 
% are in the folder.

%% After running code.  The variables can be identified
% HW_Mean_Temp is the heated wall average temperatures at each of the
% positions x y and z of multiple laser positions
% HW_SD is the heated wall standard deviation at each of the positions x y
% and z of multiple laser positions
% HW_PU is the heated wall precision uncertainty at each of the positions x
% y and z of multiple laser positions...v is used for the precision
% uncertainty claculation
%% Select files
% This section will bring in the all the temp data into your workspace
tempName={'\HeatedWallTemp.csv';'\InletTemp.csv';'\LeftWallTemp.csv';'\RightWallTemp.csv';'\TopWallTemp.csv'};% Specific Temp file names
sprintf('%s','          Select the case you want to use')
if Filetype
    folder=uigetdir('', 'Select Temperature .csv folder above the series of folders they are in');  % find folder that contains all different positions
    dirListing=dir(folder); % array of all the names of the files in the folder specified
    FlxName=uigetdir(folder,'Select Heat Flux .csv folder for the case'); %find folder that contains the flux files
    dirListingFlux=dir(fullfile(FlxName,'*csv')); % cell array of all names of the flux files in the folder
elseif ~Filetype
    tempName={'\HeatedWallTemp.csv';'\InletTemp.csv';'\LeftWallTemp.csv';'\RightWallTemp.csv';'\TopWallTemp.csv'};% Specific Temp file names
    flxName={'\HeatFlux.csv'};
    folder=uigetdir;  % find folder that contains all different positions
    dirListing=dir(folder); % array of all the names of the files in the folder specified
    dirListingFlux=dirListing;
    FlxName=folder;
else
    sprintf('%s','You need to read the introduction for this code and input a 1 or 0 for the Filetype')
end
tic
% initialize a few arrays
b=0;
for d=3:length(dirListing)
    if dirListing(d).isdir
        b=b+1;
    end
end
c=0;
if Filetype
    for d=1:length(dirListingFlux)
        if ~dirListingFlux(d).isdir
            c=c+1;
        end
    end
else
    for d=3:length(dirListingFlux)
        if ~dirListingFlux(d).isdir
            c=c+1;
        end
    end
end
fluxName=cell(1,c);
M_flux=cell(1,c);
fileName=cell(5,b);
M_temp=cell(5,b);
%% loop through files and open and save in workspace as
if Filetype
    % corresponding fileName and M_temp arrays
    j=1;
    k=1;
    for d=3:b+2
        if dirListing(d).isdir % if file  in selected file is a folder.  If it is then run if statement
            for i=1:5
                fileName(i,k)={strcat(fullfile(folder,dirListing(d).name),tempName(i))}; % full file name for each of the temp/flux files 2 below selected file
                [Num]=csvread(fileName{i,k}{1,1},1,0);
                M_temp{i,k}=Num;
                j=j+1;
            end
            k=k+1;
        else
            k=k+0;
        end
    end
    How_Many_Folders_Opened=k-1; % show how many folders in the selected file
    How_Many_Data_Temp_Sets=j-1;  % show how many temperature files you've opened
    
    % loop through for flux files
    k=1;
    for d=1:c
        if ~dirListingFlux(d).isdir % if file in selected folder is a .csv file
            fluxName(1,k)={fullfile(FlxName,dirListingFlux(d).name)};% for flux
            [Numb]=csvread(fluxName{1,k},0,0);
            M_flux{1,k}=Numb;
            k=k+1;
        end
    end
    How_Many_Flux_Files_Opened=k-1;
    clear j i k d Num ans Numb;  % clear extra variables
elseif ~Filetype
    j=1;
    k=1;
    for d=3:b+2
        if dirListing(d).isdir % if file  in selected file is a folder.  If it is then run if statement
            fluxName(1,k)={strcat(fullfile(folder,dirListing(d).name),flxName(1))};% for flux
            [Numb]=csvread(fluxName{1,k}{1,1},0,0);
            M_flux{1,k}=Numb;
            for i=1:5
                fileName(i,k)={strcat(fullfile(folder,dirListing(d).name),tempName(i))}; % full file name for each of the temp/flux files 2 below selected file
                [Num]=csvread(fileName{i,k}{1,1},1,0);
                M_temp{i,k}=Num;
                j=j+1;
            end
            k=k+1;
        else
        end
    end
    How_Many_Folders_Opened=k-1; % show how many folders in the selected file
    How_Many_Data_Temp_Sets=j-1;  % show how many temperature files you've opened
    clear j i d Num ans Numb;  % clear extra variables
else
    sprintf('%s','Seriously.... read the introduction')
end
%% Computing Temp Data next 3 sections also
% Mean temperature/Standard Deviation/Precision Uncertainty .csv files
[~,p]=size(M_temp); % size of array of temp files
[~,r]=size(M_flux); % size of array of flux files
% average temperature x y z temperature
[m,~]=size(M_temp{1,1});  % sizes of all walls
[m2,~]=size(M_temp{2,1});
[m3,~]=size(M_temp{3,1});
[m4,~]=size(M_temp{4,1});
[m5,~]=size(M_temp{5,1});
[~,n2]=size(M_flux{1,1}); % size of flux file

% initialize a few more arrays
HW_temp=zeros(length(1:m),length(1:p));
I_temp=zeros(length(1:m2),length(1:p));
LW_temp=zeros(length(1:m3),length(1:p));
RW_temp=zeros(length(1:m4),length(1:p));
TW_temp=zeros(length(1:m5),length(1:p));
HF=zeros(length(1:r),length(1:n2));
for j=1:p % loop for matrices for each wall temperatures cooresponding to positions
    HW_temp(1:m,j)=M_temp{1,j}(:,4);
    I_temp(1:m2,j)=M_temp{2,j}(:,4);
    LW_temp(1:m3,j)=M_temp{3,j}(:,4);
    RW_temp(1:m4,j)=M_temp{4,j}(:,4);
    TW_temp(1:m5,j)=M_temp{5,j}(:,4);
end
clear j;
for j=1:r
    HF(j,1:n2)=M_flux{1,j}(1,:); %and flux
end
%% average value of each wall and flux
HW_Mean_Temp=mean(HW_temp,2);
I_Mean_Temp=mean(I_temp,2);
LW_Mean_Temp=mean(LW_temp,2);
RW_Mean_Temp=mean(RW_temp,2);
TW_Mean_Temp=mean(TW_temp,2);
HF_MEAN=mean(HF);

% average temperature for each case for plots
HW_Mean_Case=mean(HW_temp);
I_Mean_Case=mean(I_temp);
LW_Mean_Case=mean(LW_temp);
RW_Mean_Case=mean(RW_temp);
TW_Mean_Case=mean(TW_temp);
%% standard deviation
HW_SD=(std(HW_temp,0,2));
I_SD=(std(I_temp,0,2));
LW_SD=(std(LW_temp,0,2));
RW_SD=(std(RW_temp,0,2));
TW_SD=(std(TW_temp,0,2));
HF_SD=std(HF);
%% precision uncertainty with 95% confidence interval = v*SD/sqrt(N)
mu=p-1; % mu is N-1
% table of mu values
v=[1:20;12.706 4.303 3.182 2.776 2.571 2.447 2.365 2.306 2.262 2.228 2.201 2.179 2.160 2.145 2.131 2.12 2.11 2.101 2.093 2.086]';
% precision uncertainty equations for each wall
HW_PU=v(mu,2)*HW_SD/sqrt(p);
I_PU=v(mu,2)*I_SD/sqrt(p);
LW_PU=v(mu,2)*LW_SD/sqrt(p);
RW_PU=v(mu,2)*RW_SD/sqrt(p);
TW_PU=v(mu,2)*TW_SD/sqrt(p);
HF_PU=v(mu,2)*HF_SD/sqrt(r);
clear j l m m2 m3 m4 m5 n n2 m6; % clear extra variables
%% Output
% write headers
% open files to write text headers to
fileIDHW=fopen(fullfile(folder,'HeatedWallTemp_Stats.csv'),'w');
fileIDI=fopen(fullfile(folder,'InletTemp_Stats.csv'),'w');
fileIDLW=fopen(fullfile(folder,'LeftWallTemp_Stats.csv'),'w');
fileIDRW=fopen(fullfile(folder,'RightWallTemp_Stats.csv'),'w');
fileIDTW=fopen(fullfile(folder,'TopWallTemp_Stats.csv'),'w');
fileIDHF=fopen(fullfile(FlxName,'HeatFlux_Stats.csv'),'w');

% print headers
Head={'X','Y','Z','T[K]','B_T[K]','S_T[K]','U_T[K]'};
Head2={'x[m]','y[m]','z[m]','q''''[W/m^2]','B_q''''[W/m^2]','S_q''''[W/m^2]','U_q''''[W/m^2]'};
fprintf(fileIDHW,'%s,%s,%s,%s,%s,%s,%s\n',Head{1,:});
fprintf(fileIDI,'%s,%s,%s,%s,%s,%s,%s\n',Head{1,:});
fprintf(fileIDLW,'%s,%s,%s,%s,%s,%s,%s\n',Head{1,:});
fprintf(fileIDRW,'%s,%s,%s,%s,%s,%s,%s\n',Head{1,:});
fprintf(fileIDTW,'%s,%s,%s,%s,%s,%s,%s\n',Head{1,:});
% fprintf(fileIDHF,'%s\t%s\t%s\t%s\t%s\n',Head2{1,:});
fprintf(fileIDHF,'%s,%s,%s,%s,%s,%s,%s\n',Head2{1,:});
% close files
fclose('all');

% First get the master arrays
HW_Master_MeanTemp_SD_PU = ...
    [M_temp{1,1}(:,1),M_temp{1,1}(:,2),M_temp{1,1}(:,3),...
    HW_Mean_Temp,ones(length(HW_Mean_Temp),1),HW_PU,sqrt(ones(length(HW_Mean_Temp),1).^2+HW_PU.^2)];
I_Master_MeanTemp_SD_PU = ...
    [M_temp{2,1}(:,1),M_temp{2,1}(:,2),M_temp{2,1}(:,3),...
    I_Mean_Temp,ones(length(I_Mean_Temp),1),I_PU,sqrt(ones(length(I_Mean_Temp),1).^2+I_PU.^2)];
LW_Master_MeanTemp_SD_PU = ...
    [M_temp{3,1}(:,1),M_temp{3,1}(:,2),M_temp{3,1}(:,3),...
    LW_Mean_Temp,ones(length(LW_Mean_Temp),1),LW_PU,sqrt(ones(length(LW_Mean_Temp),1).^2+LW_PU.^2)];
RW_Master_MeanTemp_SD_PU = ...
    [M_temp{4,1}(:,1),M_temp{4,1}(:,2),M_temp{4,1}(:,3),...
    RW_Mean_Temp,ones(length(RW_Mean_Temp),1),RW_PU,sqrt(ones(length(RW_Mean_Temp),1).^2+RW_PU.^2)];
TW_Master_MeanTemp_SD_PU = ...
    [M_temp{5,1}(:,1),M_temp{5,1}(:,2),M_temp{5,1}(:,3),...
    TW_Mean_Temp,ones(length(TW_Mean_Temp),1),TW_PU,sqrt(ones(length(TW_Mean_Temp),1).^2+TW_PU.^2)];
HF_Master_MeanTemp_SD_PU = ... % includes x coordinates for heat flux sensors
    [0.1617,0.7777,1.3936;0,0,0;0,0,0;HF_MEAN;0.05*HF_MEAN;HF_PU;sqrt((0.05*HF_MEAN).^2+HF_PU.^2)]';

% then write them to files
dlmwrite(fullfile(folder,'HeatedWallTemp_Stats.csv'),HW_Master_MeanTemp_SD_PU,'-append','precision',4);
dlmwrite(fullfile(folder,'InletTemp_Stats.csv'),I_Master_MeanTemp_SD_PU,'-append','precision',4);
dlmwrite(fullfile(folder,'LeftWallTemp_Stats.csv'),LW_Master_MeanTemp_SD_PU,'-append','precision',4);
dlmwrite(fullfile(folder,'RightWallTemp_Stats.csv'),RW_Master_MeanTemp_SD_PU,'-append','precision',4);
dlmwrite(fullfile(folder,'TopWallTemp_Stats.csv'),TW_Master_MeanTemp_SD_PU,'-append','precision',4);
dlmwrite(fullfile(FlxName,'HeatFlux_Stats.csv'),HF_Master_MeanTemp_SD_PU,'-append','precision',4);

%% Plot Average Temperature for each of the cases
% xaxis=case#   yaxis=Temp_average or HFSreading  Used to find outliers of cases
figure('Name','Heated Wall Average Temp For Each Case');
plot(1:p,HW_Mean_Case,'x','MarkerSize',12)
xlabel('Case Name'),ylabel('Average Temperature in Kelvin')
title('Heated Wall Average Temp')
set(gca,'XTick',1:p);set(gca,'XTickLabel',{dirListing(3:p+2).name},'FontSize',8)

figure('Name','Inlet Average Temp For Each Case');
plot(1:p,I_Mean_Case,'x','MarkerSize',12)
xlabel('Case Name'),ylabel('Average Temperature in Kelvin')
title('Inlet Average Temp')
set(gca,'XTick',1:p);set(gca,'XTickLabel',{dirListing(3:p+2).name},'FontSize',8)

figure('Name','Left Wall Average Temp For Each Case');
plot(1:p,LW_Mean_Case,'x','MarkerSize',12)
xlabel('Case Name'),ylabel('Average Temperature in Kelvin')
title('Left Wall Average Temp')
set(gca,'XTick',1:p);set(gca,'XTickLabel',{dirListing(3:p+2).name},'FontSize',8)

figure('Name','Right Wall Average Temp For Each Case');
plot(1:p,RW_Mean_Case,'x','MarkerSize',12)
xlabel('Case Name'),ylabel('Average Temperature in Kelvin')
title('Right Wall Average Temp')
set(gca,'XTick',1:p);set(gca,'XTickLabel',{dirListing(3:p+2).name},'FontSize',8)

figure('Name','Top Wall Average Temp For Each Case');
plot(1:p,TW_Mean_Case,'x','MarkerSize',12)
xlabel('Case Name'),ylabel('Average Temperature in Kelvin')
title('Top Wall Average Temp')
set(gca,'XTick',1:p);set(gca,'XTickLabel',{dirListing(3:p+2).name},'FontSize',8)

% Flux plot
figure('Name','Heat Flux Sensors per case')
plot(1:r,HF(:,1),'o','MarkerSize',12)
hold all
plot(1:r,HF(:,2),'d','MarkerSize',12)
plot(1:r,HF(:,3),'x','MarkerSize',12)
xlabel('Case Name'),ylabel('HFS reading in W / m^2'),title('Heat Flux Sensors')
legend('HFS 1','HFS 2','HFS 3')
if Filetype
    set(gca,'XTick',1:r);set(gca,'XTickLabel',{dirListingFlux(1:r).name},'FontSize',8)
else
    set(gca,'XTick',1:r);set(gca,'XTickLabel',{dirListingFlux(3:r+2).name},'FontSize',8)
end
%% clear extra variables
% clear fileIDHW fileIDI fileIDLW fileIDRW fileIDTW  fileIDHF ans Head Head2 p mu r FlxName HF_Mean HF_PU HF_SD HW_Mean_Temp HW_PU HW_SD
% clear I_Mean_Temp I_PU I_SD LW_Mean_Temp LW_PU LW_SD RW_Mean_Temp RW_PU RW_SD TW_Mean_Temp TW_PU TW_SD tempName b c k 
toc