function [ y0, Fit, dudy, U_dudy ] = PIV_dudy( y, u, U_u, rho, mu, yp, Fit )
%PIV_dudy, Fit PIV Data in VSL for shear stress
%   This function will calculate the velocity gradient from near-wall PIV data. It is assumed that the wall is
%   masked. Only points within the y+ specified will be used in the linear fit (data within 5-7 are generally
%   good to use). The input data should be a single streamwise velocity profile starting at y=0.

% Find wall location
% jmax = find( u == 0,1);

% jmax = find( u == 0);
% jmax = min(jmax( jmax ~= 1 )); % Remove first index

for i=2:length(u) % Find index where of first zero after non-zero values going L->R
    if u(i-1) ~= 0 && u(i) == 0
        jmax = i;
    end
end
y = y - y(jmax);

nu = mu/rho;
N = length(u);
Fit_older = Fit;
Fit_old = Fit;
for i=1:10  % Automatically select Fit for yp wall units
    %     y_fit = y(N-Fit+1:N)';
    %     u_fit = u(N-Fit+1:N)';
    y_fit = y(jmax-Fit+1:jmax)';
    u_fit = u(jmax-Fit+1:jmax)';
    U_u_fit = U_u(jmax-Fit+1:jmax)';
    
%     fitresult = fit(0.001*y_fit,u_fit,'poly1');    % Fit line to velocity near wall
%     dudy =  fitresult(1); % The gradient from the fit
%     fitresultuncert = confint(fitresult); % The confidence interval (default is 95%)
%     U_dudy_l = fitresultuncert(1,1); U_dudy_u = fitresultuncert(2,1); % The uncertainties on the gradient
    [ y0, dudy, U_dudy ] = LineFitFunc( 0.001*y_fit, u_fit, U_u_fit );
    tau = mu*dudy;   % Wall shear stress
    u_tau = sqrt(abs(tau)/rho);  % Can't do sqrt(neg)
    y_star = yp*nu/u_tau;    % Dimension of yp wall units
    Fit = N + 1 - find( y < 1000*y_star, 1 );
    if abs(Fit-Fit_old) == 0;   % Exit if the number of points doesn't change
        break;
    elseif abs(Fit-Fit_older) == 0; % Exit if oscillating around correct Fit result
        Fit = Fit_old;  break;
    end
    if Fit > 30;
        Fit = 30;
    elseif Fit == 1
        Fit = 2; % The fit needs at least two points
    end     % If using too many points, limit the number
    Fit_older = Fit_old;
    Fit_old = Fit;
end
% figure(5);
% plot(u,y,'x','LineWidth',1.5); hold on;
% xlabel('u [m/s]'); ylabel('y [mm]');
% axis([-1 5 0 5]);
% % plot([u_fit(1) u_fit(1)+dudy*(y_fit(Fit)-y_fit(1))/1000],[y_fit(1) y_fit(Fit)],'r','LineWidth',1.5);
% plot([y0 y0-dudy*(y_fit(Fit)-y_fit(1))/1000],[y_fit(Fit) y_fit(1)],'Color',[0.5 0.5 0.5],'LineWidth',1.5)
% % legend('PIV data','MATLAB Fit','Manual Fit','location','best');
% legend('PIV data','Fit','location','best');
end