clear all; close all; clc;
%% Select the csv file

[f0,p0] = uigetfile('*-uvw.csv','Select the Inlet Velocity file');

%% Calculations

Ai = xlsread([p0,f0]);
yi = Ai(:,2); zi = Ai(:,3); ui = Ai(:,4);

[z_uni,~,ic] = unique(zi); % Determine the measurement planes or unique z values
Nz = length(z_uni); % The number of planes
% Ny = length(Ai)/length(z_uni); % The number of data points in a plane
Ny = floor(length(Ai)/length(z_uni)); % The number of data points in a plane
% Yi = reshape(yi,Ny,Nz);
% Zi = reshape(zi,Ny,Nz);
% Veli = reshape(ui,Ny,Nz);

Yi = zeros(Ny,Nz);
Zi = zeros(Ny,Nz);
Veli = zeros(Ny,Nz);
for i=1:length(ui) % Contourf wants the coordinates to be in order, so organize coordinates and data
    irow = i - Ny*floor((i-1)/Ny);
    Yi(irow,ic(i)) = yi(i);
    Zi(irow,ic(i)) = zi(i);
    Veli(irow,ic(i)) = ui(i);
end

%% Plot Inlet Velocity

figure; set(gcf,'units','inches','Position',[2 6 3.5 3]);
contourf(1000*Zi,1000*Yi,Veli,50,'LineColor','none'); hold on;
for i=1:Nz % Plot measurement planes
    plot(1000*[z_uni(i) z_uni(i)],1000*[min(yi((i-1)*Ny+1:i*Ny)) max(yi((i-1)*Ny+1:i*Ny))],...
        'LineWidth',1.5,'Color',[0.5 0.5 0.5]);
end
xlabel('$z$ [mm]','Interpreter','latex'); ylabel('$y$ [mm]','Interpreter','latex'); axis equal;
h1 = colorbar; 
set(get(h1,'ylabel'),'String','$\overline{u}$ [m/s]','Interpreter','latex'); 
set(h1,'TickLabelInterpreter','latex');
ax = gca; axpos = ax.Position; cpos = h1.Position; % Get position info
cpos(3) = 0.5*cpos(3); h1.Position = cpos; ax.Position = axpos; % Change colorbar to half width
set(gca,'TickLabelInterpreter','latex','XMinorTick','on','YMinorTick','on');
set(findall(gcf,'type','axes'),'FontSize',11);
set(findall(gcf,'type','text'),'FontSize',11);
set(findall(gcf,'type','colorbar'),'FontSize',11);
tightfig2_1(gcf,-0.4,1);
