%LAST UPDATED: 10/29/2014

% Code to generate a 5-D uncertainty surface for Davis using Davis results
% from Synthetic images. 

clear all;close all; clc
tic
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% UPDATE BEFORE USE: "path_name", "key_loc", "SurfName","Sigma"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Location of the final vector files (ie. vc7) 
path_name = 'E:\NewUncertSurface\CalImages_4x32x32_75_ov_Round_2\PIV_MP(4x32x32_75%ov)\PostProc\';

%Location of Key
key_loc= 'E:\CalImages_4x32x32_75%ov_Round_2\';

%surface name (saves in Current Folder)
SurfName='Uncertainty_4x32x32_75%_Round_PostProc_2.mat';

%Select Sigma (1 or 2)
Sigma=2;

%NOTE:
%The uncertainty surface needs to have this name when to be used by the
%uncertainty code. If you wish to create and use a different surface, 
%rename the old surface and replace it with the surface you wish to use.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% NO CHANGES NECESSARY BEYOND THIS POINT %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load([key_loc 'Key.mat'])

% N=size(Key,1);
szShr=uint8(size(unique(Key(:,1)),1));
szDsp=uint8(size(unique(Key(:,2)),1));
szDia=uint8(size(unique(Key(:,3)),1));
szDen=uint8(size(unique(Key(:,4)),1));
szFo =uint8(size(unique(Key(:,5)),1));
m=uint16(0);

%Pre-allocate Arrays
%(den,dia,shr,disp)
b_k =           (zeros(szDen,szDia,szShr,szDsp,szFo));
% b_k = permute(b_k,[1,2,5,3,4]);
b_k = reshape(b_k,[szDen,szDia,szShr,szDsp,szFo]);
r_high =        b_k;
r_low =         b_k;
rr95=           b_k;
density =       b_k;
diameter =      b_k;
shear =         b_k;
displacement =  b_k;
Fo =            b_k;
p_vals =        b_k;
h =             b_k;
p =             b_k;
%Analyzed file information
Data.imzeros = '5';
Data.imext = 'vc7';
base = 'B';

VC7=readimx([path_name 'B00001.vc7']);
VecRow=size(VC7.Frames{1}.Components{1}.Planes{1},2)/2;

if Sigma==1
    sigLow=0.15865;
    sigHigh=0.84135;
elseif Sigma==2
    sigLow=0.025;
    sigHigh=0.975;
end

%Shear
for k = 1:szShr
    %Displacement
    for l = 1:szDsp
        %Diameter
        for j = 1:szDia
            %Density
            for i = 1:szDen
                %Out-of-plane motion
                for n = 1:szFo
                    m=m+1;

                    %Locate file
                    filename = [base sprintf(['%0.' Data.imzeros 'i.' Data.imext],m)];
                    %Open File
                    VC7=readimx([path_name filename]);

                    U0=VC7.Frames{1}.Components{1}.Planes{1};
                    U1=VC7.Frames{1}.Components{3}.Planes{1};
                    U2=VC7.Frames{1}.Components{5}.Planes{1};
                    U3=VC7.Frames{1}.Components{7}.Planes{1};
                    Active_Choice=VC7.Frames{1}.Components{9}.Planes{1};
                    
                    AC0=zeros(size(U0));  AC1=AC0;  AC2=AC0;  AC3=AC0;
                    AC0(Active_Choice==0)=1;
                    AC1(Active_Choice==1)=1;
                    AC2(Active_Choice==2)=1;
                    AC3(Active_Choice==4)=1;
                    
                    U=U0.*AC0 + U1.*AC1 + U2.*AC2 + U3.*AC3;
                    
                    Eval=double(VC7.Frames{1}.Components{10}.Planes{1});
                    Eval=Eval(:,VecRow);
                    
                    %Obtain vectors located along center (no displacement due to shear)
                    Vx = U(:,VecRow);
                    Vx = Vx(Eval==1);
                    
                    if size(Vx,1)==0
                        Vx=0;
                    end
                    
%                     if size(Vx,1)>2
%                     [H,P] = jbtest(Vx,0.05);
%                     h(i,j,k,l,n)=H;
%                     p(i,j,k,l,n)=P;
%                     end
                    
                    %---------------Brandon------------------------------------
                    Temp=(Vx-mean(Vx(:))).^2;
                    Temp = sort(Temp);
                    rr95(i,j,k,l,n) = Temp(ceil((sigHigh-sigLow)*length(Temp)));
                    %----------------------------------------------------------
                    
                    %Sort Values
                    Vx=sort(Vx);
                    
                    %Find Mean
                    u_bar = mean(Vx);
                    
                    %Find Number of values
                    n_vals = uint16(length(Vx));
                    
                    %Find index of r_low and r_high
                    rl = floor(sigLow*n_vals);
                    rh =  ceil(sigHigh*n_vals);
                    
                    %Populate 4-D axis values
                    density(i,j,k,l,n)  = Key(m,4);
                    diameter(i,j,k,l,n) = Key(m,3);
                    shear(i,j,k,l,n)    = Key(m,1);
                    displacement(i,j,k,l,n) = Key(m,2);
                    Fo(i,j,k,l,n)       = Key(m,5);
                    
                    %Calculate systematic uncertainty (b_k), r_high, and r_low
                    if rl==0
                    b_k(i,j,k,l,n) =  -100;
                    r_high(i,j,k,l,n) = -100; 
                    r_low(i,j,k,l,n) = -100;    
                    else
                    b_k(i,j,k,l,n) =  abs(u_bar-Key(m,2));
                    r_high(i,j,k,l,n) = Vx(rh)-u_bar; 
                    r_low(i,j,k,l,n) = u_bar-Vx(rl);
                    end
                    
                    %Number of values used
                    p_vals(i,j,k,l,n) =  n_vals;               
                end
            end
        end
    end
end
save(SurfName,'density','diameter','displacement','shear','Fo', 'b_k','r_high','r_low','rr95','p_vals','Key');%,'h','p'
toc

% %%
% for i=10:40000
%     rl = ceil(0.025*i);
%     rh =  ceil(0.975*i);
%     v_x=sort(Vx(1:i));
%     temph(i)=v_x(rh)-u_bar;
%     templ(i)=u_bar-v_x(rl);
% end
% figure(2)
% hold on
% plot(1:40000,temph)
% plot(1:40000,templ,'r')