%LAST UPDATED: 10/29/2014

function [PDia, PDen, R, DIAX, DIAY, VS, PkD, PkD2, Iavg, Corr] = DiaDen2(IM,Data,X,Y)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
%   Inputs:     X,Y - the full image                                      %
%               Data- Information from GUI                                %
%               IM  - Image                                               %
%   Output:     PDia- Particle image diameter                             %                      
%               PDen- Particle image density                              %
%               R   - X/Y Ratio of diameter                               %
%               DIAX- X Particle image diameter                           %
%               DIAY- Y Particle image diameter                           %
%               VS  - Space between Vectors (pix)                         %
%               PkD - Local max (see function Peak8)                      %
%               PkD2- Local max (see function Peak)                       %
%               Iavg- Average particle intensity                 	      %
%               Corr- Correlation peak height                             %
%               Ximg- Location for DiaDen                                 %
%               Yimg- Location for DiaDen                                 %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
szImgY=size(IM,1);
szImgX=size(IM,2);
szVy=size(X,1);
szVx=size(X,2);

%Preallocate
GS = Data.GSdia; %Grid size (pix)
VS=X(1,2)-X(1,1);%Space between Vectors (pix)
if VS==1
    dx=1;
else
    dx = GS/VS;
end
szData=ones(size(X));           %Ximg=szData;Yimg=Ximg; 
PDia=szData;    DIAX=szData;    DIAY=szData;    R=szData;
PDen=szData;    PkD=szData;     PkD2=szData;    Iavg=szData;    Corr=szData;

%If Diameter or Density are hard coded:
if str2double(Data.hardDia)==1
    PDia= str2double(Data.diam)*szData;
    R=szData;
end
if str2double(Data.hardDens)==1
   PDen=str2double(Data.dens)*szData;
   PkD= NaN*szData;
   PkD2=NaN*szData;
   Iavg=NaN*szData;
   Corr=NaN*szData;
end

%vectors per estimate value
if Data.tag==0
    vecs=dx;%2;
    if vecs==1
        dx=2;    VS=0;
    end
elseif Data.tag==1
    vecs=1; dx=2;VS=0;
end

%If either Dia or Den are not hardcoded
if str2double(Data.hardDens)+ str2double(Data.hardDens)< 2
    ii=1;
    for i = 1:vecs:szVy
        jj=1;
        for j = 1:vecs:szVx
                   
            %Upper and lower bounds for image region
            jjlow = X(1,j)-VS/2   +1;
            jjhigh = X(1,j)-VS/2 +GS;
            iilow = Y(i,1)-VS/2  +1;
            iihigh = Y(i,1)-VS/2 +GS;
            %If region is outside of image limits, place on edge
            if iihigh>szImgY
                iihigh=szImgY;
                iilow=szImgY-GS+1;
            end
            if jjhigh>szImgX
                jjhigh=szImgX;
                jjlow=szImgX-GS+1;
            end
            if iilow<1
                iihigh=GS;
                iilow=1;
            end
            if jjlow<1
                jjhigh=GS;
                jjlow=1;
            end
            
            %Image Interrogation
            img=IM(iilow:iihigh,jjlow:jjhigh);

            %Calculate Diameter values
            if str2double(Data.hardDia)==0
            [DiamAvg,Ratio,DiaX,DiaY] = Diameter(img,GS);  
            PDia(ii:ii+vecs-1,jj:jj+vecs-1) = DiamAvg;
            DIAX(ii:ii+vecs-1,jj:jj+vecs-1) = DiaX;
            DIAY(ii:ii+vecs-1,jj:jj+vecs-1) = DiaY;
            R(ii:ii+vecs-1,jj:jj+vecs-1) = Ratio;
            
            end
            
%             %Calculate Density Values
%             if str2double(Data.hardDens)==0
%             [Dens, pkDen, pkDen2, avgI, corr] = Density(img, DIAavg(ii,jj), GS);  
%             PDen(ii:ii+vecs-1,jj:jj+vecs-1) = Dens; 
%             PkD(ii:ii+vecs-1,jj:jj+vecs-1) =pkDen;
%             PkD2(ii:ii+vecs-1,jj:jj+vecs-1) =pkDen2;
%             Iavg(ii:ii+vecs-1,jj:jj+vecs-1) =avgI;
%             Corr(ii:ii+vecs-1,jj:jj+vecs-1) =corr;
%             end
            
            %Calculate Density Values
            if str2double(Data.hardDens)==0
            [DensP, pkDenP, pkDen2P, avgIP, corrP] = DensityPaper(img, PDia(ii,jj), GS);         
            PDen(ii:ii+vecs-1,jj:jj+vecs-1) = DensP; 
            PkD(ii:ii+vecs-1,jj:jj+vecs-1) =pkDenP;
            PkD2(ii:ii+vecs-1,jj:jj+vecs-1) =pkDen2P;
            Iavg(ii:ii+vecs-1,jj:jj+vecs-1) =avgIP;
            Corr(ii:ii+vecs-1,jj:jj+vecs-1) =corrP;
            end
            jj=jj+vecs;
        end
        ii=ii+vecs;
    end

%Remove NaN and negative values
PDia(isnan(PDia)|PDia<0)=0;
DIAX(isnan(DIAX)|DIAX<0)=0;
DIAY(isnan(DIAY)|DIAY<0)=0;
R(isnan(R)|R<0)=0;
PDen(isnan(PDen)|PDen<0)=0;
PkD(isnan(PkD)|PDen<0)=0;
PkD2(isnan(PkD2)|PDen<0)=0;
end

if VS~=1 %If USM
    %Crop extra ponts
    PDia=PDia(1:szVy,1:szVx);
    DIAX=DIAX(1:szVy,1:szVx);
    DIAY=DIAY(1:szVy,1:szVx);
    R=R(1:szVy,1:szVx);
    PDen=PDen(1:szVy,1:szVx); 
    PkD=PkD(1:szVy,1:szVx);
    PkD2=PkD2(1:szVy,1:szVx);
    Iavg=Iavg(1:szVy,1:szVx);
    Corr=Corr(1:szVy,1:szVx);
else
    VS=GS;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Diameter Function                                                       %
%   This function calculates the particle image diameter in both X and Y  %
%   directions as well as the average particle image diameter and ratio of%
%   DiamX/DiamY.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
function [muDiam, Ratio,DiamX,DiamY] = Diameter(img,GS)
% The relationship between the particle image diameter and the width of
% the correlation peak is discussed in Adrian (8.126)
%Subtract the minimum value of the image
img=img-min(min(img));
%Auto-correlate
A=fftn(img);
C2=real(ifftn(conj(A).*A));    
%Correlation Peak at Center of image
C2(1:GS/2,1:GS/2)=rot90(C2(1:GS/2,1:GS/2),2);
C2(1:GS/2,GS/2+1:GS)=rot90(C2(1:GS/2,GS/2+1:GS),2);
C2(GS/2+1:GS,1:GS/2)=rot90(C2(GS/2+1:GS,1:GS/2),2);
C2(GS/2+1:GS,GS/2+1:GS)=rot90(C2(GS/2+1:GS,GS/2+1:GS),2); 
if max(C2(:))>0
    %Locate Correlation Maximum
    C2=C2-min(C2(:));
    val = 1;
    
    yData=C2(GS/2-val:GS/2+val,GS/2);    
    DiamY = sqrt(-4/log(yData(1)/yData(2)));
    
    %In the Y direction
    xData2 = C2(GS/2,GS/2-val:GS/2+val);
    DiamX = sqrt(-4/log(xData2(1)/xData2(2)));
    
    DiamX(isnan(DiamX))=DiamY;
    DiamY(isnan(DiamY))=DiamX;
    %Average particle image diameter
    muDiam = (DiamX+DiamY)/2;
    
    %Calculate the diameter ratio
    if DiamY~=0
        Ratio=DiamX/DiamY;
    else
        Ratio=0;
    end
else
    muDiam = 0; DiamX=0; DiamY=0;   Ratio=0;
end

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Density Function                                                        %
% %   This function calculates the particle image density via the emperical %
% %   equation as well as peak counting. Output also includes the average   %
% %   particle intensity and the correlation peak height.                   %
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
% function [Dens, pkDen, pkDen2, avgI, CorrPeak] = Density(IA1, Dia, GS)
% %If particle counting has (dens < pkcut), the value for particle
% %ct is used 
% pkcut=3;
% %(image intensities < thresh) are set to 0 (accounts for noise)
% thresh=1;
% 
% IA3=IA1;                        % Store unalterd image
% IA1=sqrt(IA1);                   
% 
% %Locate peaks
% BWI = Peak8(IA3);%imregionalmax(IA1);
% PeaksI = IA3.*BWI;
% peaksI = PeaksI(PeaksI~=0);
% %Take the average and standard deviation of peaks (for normalizing)
% if isempty(peaksI)
% imavg = 0;
% imstd = 0;
% else
% imavg = mean(peaksI);
% imstd = std(peaksI);
% end
% %Values to normailze interrogation region
% IMmin = sqrt(min(IA3(:)));
% IMmax = sqrt(imavg+4*imstd);
% 
% %If the above value is larger than the maximum value in the image
% if IMmax > max(IA1(:))
%     IMmax = max(IA1(:));
% end
% 
% IA1=IA1-IMmin;                  % Normalize image
% IA1 = IA1*255/(IMmax-IMmin);    % Normalize image
% IA1(isinf(IA1)|isnan(IA1))=0;
% IA1(IA1>255)=255;               % Any value above IMmax = IMmax
% %---------------------------------------------------------
% %Autocorrelation peak
% A=IA1.^2;
% CorrPeak=sum(A(:));
% %----------------------------------------------------------
% %Count particles by finding local maximums
% IA4 = IA1;
% IA4(IA4<thresh)=0;
% BW2 = Peak8(IA4);
% Peaks2 = IA4.*BW2;
% peaks2 = Peaks2(Peaks2~=0);
% pk2=length(peaks2);
% pkDen =(32*32)*(pk2)/((length(IA1)-2)^2);
% 
% %Consider using peaks2 from above instead of peaks for avg intensity
% BW = Peak(IA4);
% Peaks = IA4.*BW;
% peaks = Peaks(Peaks~=0);
% pk3=length(peaks);
% pkDen2 =(32*32)*(pk3)/((length(IA1)-4)^2);
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % AVERAGE INTENSITY                                                   %
% %    Calculate average intensity by taking peak locations and stdev of%
% %    four points around them. If stdev is low then particle images is %
% %    centered on a pixel and give a better representation of actual   %
% %    particle peak intensity                                          %
% %---------------------------------------------------------------------%   
%     %Pre-allocate
%     I=zeros([size(IA1) 4]);
%     %Create 3D matrix with 3rd dimension containing the four
%     %surrounding points. (ie for pixel IA1(2,2) in image I(2,2,:) 
%     %contains: IA1(1,2) IA1(3,2) IA1(2,1) IA1(2,3)
%     I(:,2:GS,1)=IA3(:,1:GS-1);
%     I(:,1:GS-1,2)=IA3(:,2:GS);
%     I(2:GS,:,3)=IA3(1:GS-1,:);
%     I(1:GS-1,:,4)=IA3(2:GS,:);
%     %Only keep values where peaks are located. Normalize by particle hieght
%     I(:,:,1)=(I(:,:,1)./IA3).*BW;
%     I(:,:,2)=(I(:,:,2)./IA3).*BW;
%     I(:,:,3)=(I(:,:,3)./IA3).*BW;
%     I(:,:,4)=(I(:,:,4)./IA3).*BW;
%     %Remove any Nan
%     I(isnan(I))=0;
%     %Compute standard deviation of four values surronding peaks
%     Var=std(I,0,3);
%     %Calculate the max of the four values surrounding peaks
%     Imax=max(I,[],3);
%     %Create mask
%     Tmp1 = ones(size(IA1));
%     %Remove saturate particles (ie peak value = surround value)
%     Tmp1(IA3==Imax)=0;
%     %Remove peaks where surround values have large standard deviations
%     Tmp1(Var>=0.34)=0;
%     %Remove values where there are no peaks
%     Tmp1(BW==0)=0;
%     %Apply mask
%     Pks = Tmp1.*IA1;
%     %Collect peak values
%     Temp=Pks(Pks~=0);
%     
%     %Average Intensity:
%     if size(Temp,1)>1
%     avgI = mean(Temp);
%     else
%     avgI = max(IA1(:));
%     end
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %Parameters below density obtained through Least Squares fit
% %NEW SIG FIT (No background removal) (images dont have background noise)
% P=[178.296436314516	-1.4201350389567804	-2.047364206378459	-2.0321415966427034	1.4017122121945207]; %0.9933
% % %NEW SIG FIT (With background removal)
% % P=[32.78644085981989	-1.3845535365344444	-1.683670679027916	-2.027284570808351	1.3748239872603114];%0.9907
% 
% %Calculate image density
% Dens=P(1)*(CorrPeak^P(5))*(Dia^P(4))*((GS*GS)^P(2))*(avgI^P(3));
% 
% %If density is below 3 particles, resort to peak counting.
% if pkDen < pkcut
%     Dens = pkDen;
% end
% Dens(isnan(Dens))=0;    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Dens, pkDen, pkDen2, avgI, CorrPeak] = DensityPaper(IA1, Dia, GS)
%If particle counting has (dens < pkcut), the value for particle
%ct is used 
pkcut=3;
%(image intensities < thresh) are set to 0 (accounts for noise)
thresh=1;

IA3=IA1;                        % Store unalterd image
IA1=sqrt(IA1);                  % Makes for better solution  

%Locate peaks
BWI = Peak8(IA3);%imregionalmax(IA1);
PeaksI = IA3.*BWI;
peaksI = PeaksI(PeaksI~=0);
%Take the average and standard deviation of peaks (for normalizing)
if isempty(peaksI)
imavg = 0;
imstd = 0;
else
imavg = mean(peaksI);
imstd = std(peaksI);
end
%Values to normailze interrogation region
IMmin = sqrt(min(IA3(:)));
IMmax = sqrt(imavg+4*imstd);

%If the above value is larger than the maximum value in the image
if IMmax > max(IA1(:))
    IMmax = max(IA1(:));
end

IA1=IA1-IMmin;                  % Normalize image
IA1 = IA1*255/(IMmax-IMmin);    % Normalize image
IA1(isinf(IA1)|isnan(IA1))=0;
IA1(IA1>255)=255;               % Any value above IMmax = IMmax
%---------------------------------------------------------
%Autocorrelation peak
A=IA1.^2;
CorrPeak=sum(A(:));
%----------------------------------------------------------
%Count particles by finding local maximums
IA4 = IA1;
IA4(IA4<thresh)=0;
BW2 = Peak8(IA4);
Peaks2 = IA4.*BW2;
peaks2 = Peaks2(Peaks2~=0);
pk2=length(peaks2);
pkDen =(32*32)*(pk2)/((length(IA1)-2)^2);

%Consider using peaks2 from above instead of peaks for avg intensity
BW = Peak(IA4);
Peaks = IA4.*BW;
peaks = Peaks(Peaks~=0);
pk3=length(peaks);
pkDen2 =(32*32)*(pk3)/((length(IA1)-4)^2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% AVERAGE INTENSITY                                                   %
%    Calculate average intensity by taking peak locations and stdev of%
%    four points around them. If stdev is low then particle images is %
%    centered on a pixel and give a better representation of actual   %
%    particle peak intensity                                          %
%---------------------------------------------------------------------%   
    %Pre-allocate
    I=zeros([size(IA1) 4]);
    %Create 3D matrix with 3rd dimension containing the four
    %surrounding points. (ie for pixel IA1(2,2) in image I(2,2,:) 
    %contains: IA1(1,2) IA1(3,2) IA1(2,1) IA1(2,3)
    I(:,2:GS,1)=IA3(:,1:GS-1);
    I(:,1:GS-1,2)=IA3(:,2:GS);
    I(2:GS,:,3)=IA3(1:GS-1,:);
    I(1:GS-1,:,4)=IA3(2:GS,:);
    %Only keep values where peaks are located
    I(:,:,1)=(I(:,:,1)./IA3).*BW;
    I(:,:,2)=(I(:,:,2)./IA3).*BW;
    I(:,:,3)=(I(:,:,3)./IA3).*BW;
    I(:,:,4)=(I(:,:,4)./IA3).*BW;
    %Remove any Nan
    I(isnan(I))=0;
    %Compute standard deviation of four values surronding peaks
    Var=std(I,0,3);
    %Calculate the max of the four values surrounding peaks
    Imax=max(I,[],3);
    %Create mask
    Tmp1 = ones(size(IA1));
    %Remove saturate particles (ie peak value = surround value)
    Tmp1(IA3==Imax)=0;
    %Remove peaks where surround values have large standard deviations
    Tmp1(Var>=0.34)=0;
    %Remove values where there are no peaks
    Tmp1(BW==0)=0;
    %Apply mask
    Pks = Tmp1.*IA1;
    %Collect peak values
    Temp=Pks(Pks~=0);
    
%     %Average Intensity:
%     if size(Temp,1)>1
%     avgI = mean(Temp);
%     elseif length(peaks)>=1
%         pks=peaks(peaks>=25);
%     avgI = mean(pks);
%     else
%     avgI = max(IA3(:));
%     end

%Average (Intensity^2):
    if size(Temp,1)>1
    avgI = mean(Temp.^2);
    elseif length(peaks)>=1
        pks=peaks(peaks>=25);
    avgI = mean(pks.^2);
    else
    avgI = max(IA3(:)).^2;
    end
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
%NEWER SIG FIT (intensity squared before averaged
xx=[48.186708912451031  -1.345657215330582  -0.839963622392766  -1.992010438118426   1.330159735343349]; %0.9859

a=xx(1);
b=xx(5);
c=xx(4);
d=xx(2);
e=xx(3);

%Calculate image density
Dens=a*(CorrPeak^b)*(Dia^c)*((GS*GS)^d)*(avgI^e);
Dens(Dia==0)=0;
%If density is below 3 particles, resort to peak counting.
if pkDen < pkcut
    Dens = pkDen;
end
Dens(isnan(Dens))=0;    






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Peak Finder                                                             %
%   This function locates local maximums by comparing each location (i,j) %
%   to the surrounding 20 points                                          %
%                            _ _ _                                        %
%                          _|_|_|_|_                                      %
%                         |_|_|_|_|_|                                     %
%                       i |_|_|x|_|_|                                     %
%                         |_|_|_|_|_|                                     %
%                           |_|_|_|                                       %
%                              j                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Matrx = Peak(IMG)
    
ny = size(IMG,2);
nx = size(IMG,1);
Matrx = zeros(size(IMG));
for i = 3:nx-2
    for j = 3:ny-2
        if         IMG(i,j)> IMG(i,j-2) && IMG(i,j)> IMG(i,j-1)...
                && IMG(i,j)> IMG(i,j+1) && IMG(i,j)> IMG(i,j+2) && IMG(i,j)> IMG(i-1,j-1)...
                && IMG(i,j)> IMG(i-1,j) && IMG(i,j)> IMG(i-1,j+1)&&IMG(i,j)> IMG(i-2,j)...
                && IMG(i,j)> IMG(i+1,j-1)&&IMG(i,j)> IMG(i+1,j) && IMG(i,j)> IMG(i+1,j+1)...
                && IMG(i,j)> IMG(i+2,j)...
                && IMG(i,j)> IMG(i-1,j-2)&& IMG(i,j)> IMG(i+1,j-2)...
                && IMG(i,j)> IMG(i+2,j-1)&& IMG(i,j)> IMG(i+2,j+1)&& IMG(i,j)> IMG(i-1,j+2)...
                && IMG(i,j)> IMG(i+1,j+2)&& IMG(i,j)> IMG(i-2,j+1)&& IMG(i,j)> IMG(i-2,j-1)
            Matrx(i,j) = 1;
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Peak Finder 8                                                           %
%   This function locates local maximums by comparing each location (i,j) %
%   to the surrounding 8 points                                           %
%                            _ _ _                                        %
%                           |_|_|_|                                       %
%                         i |_|x|_|                                       %
%                           |_|_|_|                                       %
%                              j                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                             
function Matrx = Peak8(IMG)
ny = size(IMG,2);
nx = size(IMG,1);
Matrx = zeros(size(IMG));
for i = 2:nx-1
    for j = 2:ny-1
        if         IMG(i,j)> IMG(i,j-1) && IMG(i,j)> IMG(i,j+1) && IMG(i,j)> IMG(i-1,j-1)...
                && IMG(i,j)> IMG(i-1,j) && IMG(i,j)> IMG(i-1,j+1)...
                && IMG(i,j)> IMG(i+1,j-1)&&IMG(i,j)> IMG(i+1,j) && IMG(i,j)> IMG(i+1,j+1)
            Matrx(i,j) = 1;
        end
    end
end