%LAST UPDATED: 10/29/2014

% Update History:
% 3-18-13   Included X and Y pixel locations in output file
% 4-4-13    Added background feature for single image

% USE THIS CODE BY RUNNING THE GUI 'PIVdiadenGUI.m' WITH 
% 'PIVdiadenGUI.fig' IN THE SAME LOCATION AS THIS CODE.
%
% TO USE THIS CODE WITHOUT THE GUI THE STRUCTURE 'Data' MUST 
% BE CREATED TO RUN 'PIVdiaden(Data)'.
%{
      Data.imdirec = 'C:\Users\EFDL\Downloads\repivdiadencode';
      Data.imbase  = 'B';
      Data.imzeros = '5';
      Data.imext   = 'tif';
      Data.imcstep = '0';
      Data.imfstep = '1';
      Data.imfstart= '1';
      Data.imfend  = '6';
      Data.imNum   = '1';
      Data.DiaGS = '2';
      Data.DenGS = '2';
      Data.dens = ''; 
      Data.diam = '';
      Data.hardDens = '0';
      Data.hardDia = '0';
      Data.Plot = '0';
      Data.outbase = 'B00001';
      Data.direcout= 'C:\Users\EFDL\Desktop\';
      Data.par     = '0';
      Data.parprocessors = '6';
      PIVdiaden(Data)
%}

function PIVdiaden(Data)
% Uncertainty analysis tool based upon input structure "Data"
% generated using the PIVuncertainty.m and PIVuncertainty.fig GUI file.
% 
% Required GUI inputs include: 
%     -Image files (including DaVis *.im7 files)
% Data Structure includes:
%     -imdirec - directory where images are located
%     -imbase  - base letter(s) of image (ie. 'B' for 'B00001.im7')
%     -imzeros - number of number places in file(ie 'B00001.im7' has 5)
%     -imext   - extension on image file (ie. *.im7, *.png, etc.)
%     -imcstep - step from first image to second image
%     -imfstep - step between first and third images
%     -imfstart- first image number
%     -imfend  - last image number
%     -imNum   - chose first or second image (ie. *.im7 file)
%     -DiaGS   - Diameter Grid size (ie. 128x128, 64x64)
%     -DenGS   - Density Grid size (ie. 128x128, 64x64)
%     -dens    - Image density if hard coded (leave as '' otherwise) 
%     -diam    - Image diameter if hard coded (leave as '' otherwise) 
%     -hardDens- Hard code density '0' = no, '1' = yes
%     -hrdDia  - Hard code diameter '0' = no, '1' = yes
%     -Plot    - Plot results '0' = no, '1' = yes
%     -outbase - output file name (ie. B00001)
%     -outdirec- directory where results are saved
%     -par     - run in parallel
%     -parprocessors - number of processors to use
%     -bkgd    - minimum pixel value over all images
%     -imgMean - mean pixel value over all images
%     -imgStd  - pixel standard deviation over all images
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Data.imfstart=str2double(Data.imfstart);
Data.imfstep=str2double(Data.imfstep);
Data.imfend=str2double(Data.imfend);
Data.imNum=str2double(Data.imNum);
Data.par=str2double(Data.par);
Data.parprocessors=str2double(Data.parprocessors);

fprintf(['Current Job: ' Data.batchname '\n'])

%Convert Grid Size selection to pixel values
if Data.DiaGS=='1'
    Data.GSdia=32;
elseif Data.DiaGS=='2'
    Data.GSdia=64;
elseif Data.DiaGS=='3'
    Data.GSdia=128;
elseif Data.DiaGS=='4'
    Data.GSdia=256;
end

if Data.bkgd =='1'
%% Calculate BackGround:
%{
    Noise is introduced through stationary objects, reflections, and a 
    non-zero backgrond. The background is calculated by comparing all 
    images in the set and taking the minimum value for each pixel.
%}
    fprintf('\n----------- Calculating Background Image  ------------\n')
    %Image file location and base (ie. C:\Folder\File-prefix)
    if ispc
        imbase=[Data.imdirec '\' Data.imbase];
    else
        imbase=[Data.imdirec '/' Data.imbase];
    end

    %Image sets
    I1 = Data.imfstart:Data.imfstep:Data.imfend;

    if strcmp(Data.imext,'im7')==1 || strcmp(Data.imext,'IM7')==1
        %Load single image to determin image size
        Davis=readimx([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I1(1))]);
        im2=double(Davis.Frames{Data.imNum}.Components{1}.Planes{1}');
    else
        %All other image files (Same method as for DaVis *im7 files [above])
        im2=double(imread([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I1(1))]));
    end

    %Pre-allocate matrices for the background and to calculate the mean/stdv
    Data.bkgspc=length(I1);
    bkgd = ones([size(im2),2])*(2^16);
    bkgrd2=zeros([size(im2), ceil(length(I1)/Data.bkgspc)]);
    %Load every image pair 
    j=0;
    Imax=0;
    for q=1:length(I1)
        if strcmp(Data.imext,'im7')==1 || strcmp(Data.imext,'IM7')==1
            %DaVis Image File
            Davis=readimx([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I1(q))]);
            im2=double(Davis.Frames{Data.imNum}.Components{1}.Planes{1}');
        else
            %All other image files (Same method as for DaVis *im7 files [above])
            im2=double(imread([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I1(q))]));
        end
        %Assign current image to 2nd slot in matrix
        bkgd(:,:,2) = im2;
        %Compare the current min the the new image, keep lowest value
        bkgd(:,:,1) = min(bkgd,[],3);

        %Sum the individual values and squared values for mean and stdv
        % imgSum = imgSum + im2;
        % imgSum2 = imgSum2 + im2.^2;
        Imax=max(Imax,max(im2(:)));
        if (q/Data.bkgspc-floor(q/Data.bkgspc))==0 || q==length(I1)
            j=j+1;
            bkgrd2(:,:,j)= bkgd(:,:,1);
            bkgd = ones([size(im2),2])*(2^16);
        end
    end
    %Store background in Data structure for future use
    Data.bkgrd = bkgrd2;%bkgd(:,:,1);

        
%     %Calculate and store mean
%     Data.imgMean = (imgSum./q);
%     %Calculate and store standard deviation using: 
%     %http://en.wikipedia.org/wiki/Computational_formula_for_the_variance
%     Data.imgStd = sqrt(imgSum2./(q-1)-(q/(q-1))*((imgSum./q).^2));
    Data.Imax=Imax;
        
        if Data.SaveBkgd=='1'
            bitdepth = ceil(log(Imax)/log(2));

            if ispc
            Data.outdirec= [Data.outdirec '\'];
            else
            Data.outdirec= [Data.outdirec '/'];
            end
           imwrite(Data.bkgrd/(2^16),[Data.outdirec 'BkgdImg.png'],'bitdepth',16) 
           %Before and After images
           imwrite(im2/(2^bitdepth),[Data.outdirec 'BeforeBkgdSub.png'],'bitdepth',16)
           imwrite((im2-Data.bkgrd)/(2^bitdepth),[Data.outdirec 'AfterBkgdSub.png'],'bitdepth',16)
        end
end

%Uncomment to make the following plots:
%         map = gray(2^bitdepth);
%         %Shows Original Image
%         figure(1)
%         imshow(im2,map)   
%         title('Original Image')
% 
%         %Shows time minimum background
%         figure(2)
%         imshow(Data.bkgrd,map) 
%         title('Background Image')
%         %Shows Figure (1) without background
%         figure(3)
%         Temp2=im2-Data.bkgrd;
%         Temp2(Temp2<0)=0;
%         imshow(Temp2,map) 
%         title('Original Image with Background subtracted')
%         % %}
%         %End Calculage Backgroud
    


%% Run Jobs in Parallel
%{
    If the "Run in Parallel" box is checked in the PIVuncertainty GUI, 
    this section opens matlabpool which enables parallel processing.
    Images are devided between the total number of processors.
%}
if Data.par
    mkdir('TempFolder')
    fprintf('\n--- Initializing Processor Cores for Parallel Job ----\n')
    poolopen=1;
    
    %Don't open more processors than there are image pairs
    if length(Data.imfstart:Data.imfstep:Data.imfend) < (Data.parprocessors)
        Data.parprocessors=num2str(length(Data.imfstart:Data.imfstep:Data.imfend));
    end
    
    try
        %Open Matlab Pool
%         matlabpool('open','local',Data.parprocessors);
        parpool('local',Data.parprocessors);
    catch
        try
            %If First attepmt doesn't work, close and retry
%             matlabpool close
            delete(gcp('nocreate'));
%             matlabpool('open','local',Data.parprocessors);
            parpool('local',Data.parprocessors);
        catch
            %If Second attempt doesn't work, continue with single processor
            beep
            disp('Error Running Job in Parallel - Defaulting to Single Processor')
            poolopen=0;
            fprintf('\n-------------- Processing Dataset ------------------\n')
            diadenprocessing(Data)
            fprintf('---------------- Job Completed ---------------------\n')
        end
    end
    if poolopen
        %Pool successfully open, divide sets
        I1=Data.imfstart:Data.imfstep:Data.imfend;

            fprintf('\n--------------- Processing Dataset -------------------\n')
            spmd
                verstr=version('-release');
                if str2double(verstr(1:4))>=2010
                    I1dist=getLocalPart(codistributed(I1,codistributor('1d',2)));
                else
                    I1dist=localPart(codistributed(I1,codistributor('1d',2),'convert'));
                end
                diadenprocessing(Data,I1dist);
            end
            fprintf('----------------- Job Completed ----------------------\n')
            
        if poolopen
%             matlabpool close
            delete(gcp('nocreate'));
        end
    end
    %Combine solution from the parallel processors
    if Data.par
    DiadenCombine(Data)
    end
else
    %"Run in Parallel" not selected - Default to single processor
    fprintf('\n-------------- Processing Dataset ------------------\n')
    diadenprocessing(Data)
    fprintf('---------------- Job Completed ---------------------\n')
end


function diadenprocessing(Data,I1)
    
tic
% Image file location/base (ie. C:\Folder\File-prefix) and output directory
if ispc
    imbase=[Data.imdirec '\' Data.imbase];
    tempout = ['TempFolder\' Data.imbase];
    Data.outdirec= [Data.outdirec '\'];
else
    imbase=[Data.imdirec '/' Data.imbase];
    tempout = ['TempFolder/' Data.imbase];
    Data.outdirec= [Data.outdirec '/'];
end

if nargin==1 %(Not runing in parallel)
%Image indices
I1 = (Data.imfstart):(Data.imfstep):(Data.imfend);
end


%% EVALUATE IMAGE PAIRS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for q=1:length(I1)
    
    %LOAD IMAGE PAIRS
    if strcmp(Data.imext,'im7')==1 || strcmp(Data.imext,'IM7')==1
        %DaVis Image File
        Davis=readimx([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I1(q))]);
        im2=double(Davis.Frames{Data.imNum}.Components{1}.Planes{1}');
    else
        %All other image files (bmp, gif, jpg, png, tif, etc)
        im2=double(imread([imbase sprintf(['%0.' Data.imzeros 'i.' Data.imext],I1(q))]));
    end

%     x = ReplicatePad(1:size(im2,2), [ size(im2,1)-1 0]);
    Data.tag=1;
    Data.ov=0.75;
    X=Data.GSdia*0.5*(1-Data.ov):Data.GSdia*(1-Data.ov):size(im2,2);
    Y=Data.GSdia*0.5*(1-Data.ov):Data.GSdia*(1-Data.ov):size(im2,1);
    [X,Y]=meshgrid(X,Y);
    
    %Subtract background and 20 to compensate for an low level noise
    %(Effects on the image densitsy are minimal (tipically 0.5% of max)
    if Data.bkgd =='1'
        bitdepth = ceil(log(Data.Imax)/log(2));
        if bitdepth <= 16
            im2 = im2-Data.bkgrd-(0.0048828125*2^bitdepth);
        else
            im2 = im2-Data.bkgrd-20;
        end
    end
        
    %Any negative values set to zero
    im2(im2<0)=0;
    
    %ESTIMATE THE FOLLOWING VALUES:
    %   P_Dia       - Particle image diameter
    %   P_Dens      - Image density
    %   DiaRatio    - (dia_x/dia_y) to detect elliptical particles
    %   PeakDen     - Image density using peak counting (8-neighbor)
    %   PeakDen2    - Image density using peak counting (20-neighbors)
    %   Iavg        - Average particle intensity
    [P_Dia, P_Dens, DiaRatio,DiaX,DiaY,~, PeakDen, PeakDen2, Iavg, Corr]=DiaDen2(im2,Data,X,Y);

    %Convert to single precision to save memory
    P_Dia=single(P_Dia);
    P_Dens=single(P_Dens);
    DiaRatio=single(DiaRatio);
    PeakDen=single(PeakDen);
    PeakDen2=single(PeakDen2);
    Iavg=single(Iavg);
    Corr=single(Corr);
    X=single(X);
    Y=single(Y);
    DiaX=single(DiaX);
    DiaY=single(DiaY);
    %If running in parallel, save individual files (combined ln 248) 
    if Data.par    
        save([tempout sprintf(['%0.' Data.imzeros 'i.mat'],I1(q))],...
            'P_Dia','P_Dens','DiaRatio','PeakDen','PeakDen2','Iavg','Corr','X','Y','DiaX','DiaY');
    else
    %Otherwise save results for each image to matrix    
        avgI(:,:,q)=Iavg;
        Cor(:,:,q)=Corr;
        Dia(:,:,q)=P_Dia;
        Den(:,:,q)=P_Dens;
        Rat(:,:,q)=DiaRatio;
        PkDen(:,:,q)=PeakDen;
        PkDen2(:,:,q)=PeakDen2;
        Diax(:,:,q)=DiaX;
        Diay(:,:,q)=DiaY;
    end
    %Output current step
     fprintf(['Completed (' num2str(q) '/' num2str(length(I1)) ')\n'])
end
%If not running in parallel, continue to stats subfunction
if Data.par == 0
    DiaDenMeanStdev(Data,Dia,Den,Rat,PkDen,PkDen2,avgI,Cor,X,Y,Diax,Diay)
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% When code PIVdiaden is run in parallel individual image solutions   %
% are stored and then read in by this function. The individual results%
% are stored in a temporary file that is deleted at the end of this   %
% function.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function DiadenCombine(Data)

%Determine PC or Mac
if ispc
    tempout = ['TempFolder\' Data.imbase];
    Data.outdirec= [Data.outdirec '\'];
else
    tempout = ['TempFolder/' Data.imbase];
    Data.outdirec= [Data.outdirec '/'];
end

%Number of Images processed
N = length(Data.imfstart:Data.imfstep:Data.imfend);

%Open first mat file to preallocate size
fname = [tempout sprintf(['%0.' Data.imzeros 'i.mat'],Data.imfstart)];
load(fname)
%Preallocate matrices
Dia = zeros(size(P_Dia,1),size(P_Dia,2),N,'single');
Den = zeros(size(P_Dens,1),size(P_Dens,2),N,'single');
Rat = zeros(size(P_Dia,1),size(P_Dia,2),N,'single');
PkD = zeros(size(P_Dens,1),size(P_Dens,2),N,'single');
PkD2 = zeros(size(P_Dens,1),size(P_Dens,2),N,'single');
avgI = zeros(size(P_Dens,1),size(P_Dens,2),N,'single');
Cor = zeros(size(P_Dens,1),size(P_Dens,2),N,'single');
Diax = zeros(size(P_Dens,1),size(P_Dens,2),N,'single');
Diay = zeros(size(P_Dens,1),size(P_Dens,2),N,'single');
%Load all *.mat files and store in three dimensional array
n = 1;
for i = Data.imfstart:Data.imfstep:Data.imfend
    fname = [tempout sprintf(['%0.' Data.imzeros 'i.mat'],i)];
    load(fname)
    Dia(:,:,n) = P_Dia;
    Den(:,:,n) = P_Dens;
    Rat(:,:,n) = DiaRatio;
    PkD(:,:,n) = PeakDen;
    PkD2(:,:,n)= PeakDen2;
    avgI(:,:,n)= Iavg;
    Cor(:,:,n)= Corr;
    Diax(:,:,n)=DiaX;
    Diay(:,:,n)=DiaY;
    n = n + 1;   
end
%Calculate Mean and Standard deviation
DiaDenMeanStdev(Data,Dia,Den,Rat,PkD,PkD2,avgI,Cor,X,Y,Diax,Diay)
%Delete Temporary Folder
rmdir('TempFolder','s')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This portion calculates the mean and standard deviation of the      %
% entire image set. Used for both parallel and serial cases.          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function DiaDenMeanStdev(Data,Dia,Den,Ratio,PkDen,PkD2,Iavg,Cor,X,Y,Diax,Diay)
%Remove NaNs
Dia(isnan(Dia))=0;
Den(isnan(Den))=0;
Ratio(isnan(Ratio))=0;
Cor(isnan(Cor))=0;
PkDen(isnan(PkDen))=0;
PkD2(isnan(PkD2))=0;
Iavg(isnan(Iavg))=0;
Diax(isnan(Diax))=0;
Diay(isnan(Diay))=0;
%Calculate Mean and Standard Deviation
Dia_mu = mean(Dia,3);
Den_mu = mean(Den,3);
Ratio_mu = mean(Ratio,3);
PkDen_mu = mean(PkDen,3);
PkD2_mu = mean(PkD2,3);
Iavg_mu = mean(Iavg,3);
Diax_mu = mean(Diax,3);
Diay_mu = mean(Diay,3);

Den_stdv=std(Den,0,3);
Dia_stdv=std(Dia,0,3);
Ratio_stdv=std(Ratio,0,3);
PkDen_stdv=std(PkDen,0,3);
PkD2_stdv=std(PkD2,0,3);

temp = ' ';
%Save Data to specified location
save([Data.outdirec Data.outbase],'Dia',  'Dia_mu', 'Dia_stdv','Den', 'Den_mu', 'Den_stdv','X','Y','Diax_mu','Diay_mu');
if str2double(Data.OutLM)
    save([Data.outdirec Data.outbase],'PkDen', 'PkDen_mu', 'PkDen_stdv','-append');
end
if str2double(Data.OutRatio)
    save([Data.outdirec Data.outbase],'Ratio', 'Ratio_mu', 'Ratio_stdv','-append');
end
if str2double(Data.OutCorr)
    save([Data.outdirec Data.outbase],'Cor','-append');
end

%Plot if desired
if Data.Plot == '1'
figure
contourf(double(Dia_mu),30,'DisplayName','DIA','edgecolor','none');
colorbar;title('Particle Image Diameter')
figure 
contourf(double(Den_mu),30,'DisplayName','pd','edgecolor','none');
colorbar;title('Particle Density particles/(32x32)')
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% %%% ReplicatePad
% function b = ReplicatePad(a, padSize)
% numDims = numel(padSize);
% idx   = cell(1,numDims);
% for k = 1:numDims
%   M = size(a,k);
%   p = padSize(k);
%   onesVector = ones(1,p);
%   idx{k}   = [1:M M*onesVector];
% end
%  b = a(idx{:});