clear all; close all; clc;

% This code can be run on the Stats.mat output from PIVUncertainty for the v velocity with uncertainty bands.
%
% Blake Lance
% 12 September 2014

uiopen('matlab');

tic;

proloc = round(0.5*size(v_ave,1));

% Plot vertical velocity and its uncertainty
figure(1); errorbar(X(proloc,:),v_ave(proloc,:),Uv(proloc,:));
xlabel('x [mm]');   ylabel('v [m/s]');  axis tight;

% Plot \overline{v'v'} and its uncertainty
figure(2);  errorbar(X(proloc,:),vvt(proloc,:),Uvv_minus(proloc,:),Uvv_plus(proloc,:));
xlabel('x [mm]');   ylabel('$\overline{v''v''} [m^2/s^2]$','interpreter','latex');  axis tight;

% Plot partical density
figure(3);  plot(X(proloc,:),Den_ave(proloc,:));
xlabel('x [mm]');   ylabel('Particle Density [particles/32x32]');  axis tight;

% Plot partical diameter
figure(4);  plot(X(proloc,:),Dia_ave(proloc,:));
xlabel('x [mm]');   ylabel('Particle Diameter [pixels]');  axis tight;

% Plot shear
figure(5);  plot(X(proloc,:),dvdx_ave(proloc,:));
xlabel('x [mm]');   ylabel('Shear [pixels/pixel]');  axis tight;

% Plot streamwise velocity and u'v'
figure; h1 = axes('Position',[0.09 0.1 0.8 0.8]);
[hAx,hLine1,hLine2] = plotyy(X(proloc,:),-v_ave(proloc,:),X(proloc,:),uvt(proloc,:));
xlabel('y [mm]');   ylabel(hAx(1),'$\rm{\overline{u} \; [m/s]}$','interpreter','latex'); axis(hAx(1),'tight');
ylabel(hAx(2),'$\rm{\overline{u''v''} \; [m^2/s^2]}$','interpreter','latex'); axis(hAx(2),'tight');
fontname = 'CMU Serif Roman';
set(0,'defaultaxesfontname',fontname);
set(0,'defaulttextfontname',fontname);

toc;
