% Code to interpolate the Temperature data to a fine grid for input to Star-CCM+

close all; clear all; clc;
tic;
%% Read in the files to be interpolated.
%  Open the velocity, epsilon, and k files
sprintf('%c','Select the .csv file for the inlet temperature:')
[f1,p] = uigetfile('.csv');  % Ask user to choose velocity file
[inlet]=xlsread([p,f1]);
inletT=inlet(:,4);
inletz=inlet(:,3);
inlety=inlet(:,2);

sprintf('%c','Select the .csv file for the plate temperature:')
[f1,p] = uigetfile('.csv');  % Ask user to choose velocity file
[plate]=xlsread([p,f1]);
plateT=plate(:,4);
platex=plate(:,1);
platez=plate(:,3);

%% Define the Fine Grid for the plate
platex_fine=[0:0.001:1.859]';
platez_fine=[-0.1507:0.001:0.1507]';

% and for the inlet temperature
inlety_fine=[0.076:0.0005:.229]';
inletz_fine=[-0.102:0.0005:0.102]';

%% Interpolate the data to a fine grid
Fu = TriScatteredInterp(inlety,inletz,inletT);
[qx,qy] = meshgrid(inlety_fine,inletz_fine);
InletT = Fu(qx,qy);

Fv = TriScatteredInterp(platex,platez,plateT);
[qx,qy] = meshgrid(platex_fine,platez_fine);
PlateT = Fv(qx,qy);

%% Sort to write out the files

TempI=zeros(size(InletT,1)*size(InletT,2),4);
TempPlate=zeros(size(PlateT,1)*size(PlateT,2),4);

count=0;
for i=1:size(InletT,1)
    for j=1:size(InletT,2)
        count=count+1;
        TempI(count,:)=[0,inlety_fine(j),inletz_fine(i),InletT(i,j)];
    end
end

count=0;
for i=1:size(PlateT,1)
    for j=1:size(PlateT,2)
        count=count+1;
        TempPlate(count,:)=[platex_fine(j),0,platez_fine(i),PlateT(i,j)];
    end
end
smallTP=min(TempPlate);
TempPlate(isnan(TempPlate))=smallTP(4);

%% Write the inlet temperature file
header=['X,Y,Z,Temp,'];
outid = fopen('Inlet-Temp-interp.csv', 'w+');
fprintf(outid, '%s', header);
fprintf(outid, '\n');
fclose(outid);

dlmwrite('Inlet-Temp-interp.csv',TempI,'delimiter',',','precision',7,'-append')

header=['X,Y,Z,Temp,'];
outid = fopen('HeatedWall-Temp-interp.csv', 'w+');
fprintf(outid, '%s', header);
fprintf(outid, '\n');
fclose(outid);

dlmwrite('HeatedWall-Temp-interp.csv',TempPlate,'delimiter',',','precision',7,'-append')
toc;