function [imin,imax,jmin,jmax] = TrimToMask( U )

% This subroutine takes any 2D velocity field U and finds the i and j
  % indeces of the last zeros around the perimeter of the masked area.
  %     Remember:   the i axis is aligned with x increasing to right
  %                 the j axis is reverse with y increasing to bottom
  
% Determine left-hand mask boundary
jmin = zeros(1,size(U,1));
for i=1:size(U,1)   % Run through rows
    trigger = 0;    % Reset trigger for every new row
    for j=1:size(U,2)   % Run through columns from left to right
        if trigger == 0 % The first non-zero hasn't been found
            if U(i,j) == 0.
                jmin(i) = j;    % x-index of last 0.
            else
                trigger = 1;    % set trigger for first non-zero
            end
        end    
    end
end

% Determine right-hand mask boundary
jmax = ones(1,size(U,1))*size(U,1);
for i=1:size(U,1)   % Run through rows
    trigger = 0;    % Reset trigger for every new row
    for j=size(U,2):-1:1   % Run through columns from right to left
        if trigger == 0 % The first non-zero hasn't been found
            if U(i,j) == 0.
                jmax(i) = j;  % x-index of last 0.
            else
                trigger = 1;    % set trigger for first non-zero
            end
        end    
    end
end

% Determine top mask boundary
imax = zeros(1,size(U,2));
for j=1:size(U,2)   % Run through columns
    trigger = 0;    % Reset trigger for every new column
    for i=1:size(U,1)   % Run through rows from top to bottom
        if trigger == 0 % The first non-zero hasn't been found
            if U(i,j) == 0.
                imax(j) = i;  % y-index of last 0.
            else
                trigger = 1;    % set trigger for first non-zero
            end
        end    
    end
end

% Determine bottom mask boundary
imin = zeros(1,size(U,2));
for j=1:size(U,2)   % Run through columns
    trigger = 0;    % Reset trigger for every new column
    for i=size(U,1):-1:1   % Run through rows from bottom to top
        if trigger == 0 % The first non-zero hasn't been found
            if U(i,j) == 0.
                imin(j) = i;  % y-index of last 0.
            else
                trigger = 1;    % set trigger for first non-zero
            end
        end
    end
end

imin = max(imin);
imax = min(imax);
jmin = min(jmin);
jmax = max(jmax);